c---+----1----+----2----+----3----+----4----+----5----+----6----+----7--
c
c>>>>>>>>>>>>>>>>>>>>> EXTERNALLY CALLED ROUTINES <<<<<<<<<<<<<<<<<<<<<<
c
c  To use the capabilities of this library, the user would only call the 
c  following three routines in their program.  All subsequent routines 
c  are included to satisfy all the calls that the following three 
c  routines make.
c
c      GRFXOPN    opens the plot file
c      GRFX1D     makes a single page, multiple-frame, single variable 
c                 per frame plot
c      GRFXCLS    closes the plot file.
c
c  Consult the headers of each of these routines to determine the 
c  required syntax and purpose of each variable in the argument list.
c
c  As an example of a call to GRFX1D:
c
c      call grfx1d ( psi, den, 0, 1, npnt, 1, 1, 0.0, 0.0, 0.0, 0.0
c    1             , 2, 1.0, 1, 2, 2, 1, 2, 1, xlab, ylab )
c
c  creates a square linear-linear graph of all "npnt" points of the 
c  dependent variable "den" (density) plotted against the independent 
c  variable "psi", allowing the plotter to determine the optimal 
c  plotting window, joining all points with lines.  The last six 
c  integers the plotter to arrange the graphs in a 2 x 2 fashion on a 
c  single page (rather than one plot per page).
c
c  The last two parameters, "xlab" and "ylab" use something called "NCAR 
c  mark-up language" to label the plots.  You're perfectly safe to just 
c  set xlab='x' and ylab='rho' here, though if you wanted to be a little 
c  fancy, try: xlab = '|F10|x/t', ylab = '|F5|r'.
c
c=======================================================================
c
       subroutine grfx1d ( xvar, yvar, yexp, ibeg, iend, linlog, imm
     1                   , xmn, xmx, ymn, ymx, corl, aspect, allxl
     2                   , allyl, np1h, ip1h, np1v, ip1v, xlab, ylab )
c
c    dac:grfx03.grfx1d <-------------------------------- plots 1-d slice
c                                                             june, 1990
c
c    written by: David Clarke
c    modified 1: June, 1996 by David Clarke; rewritten to generate
c                publication quality plots.
c    modified 2: December, 1996 by David Clarke; added multiple frames
c                per page capability.
c    modified 3: October, 2003 by Alexander Men'shchikov; generalized to
c                include the PSPLOT graphics library.
c    modified 4: February, 2005 by David Clarke; streamlined Sasha's
c                modifications
c    modified 5: April, 2006 by DAC; introduced "yexp", in analogy to 
c                "qexp" and "vexp" in GRFX2D.
c    modified 6: December, 2016 by DAC; simplified from July, 2015 
c                version for this "cheap and cheerful" standalone 
c                library to do single-variable, multi-framed plots on a 
c                single page.
c
c  PURPOSE: This is a pared down version of the subroutine by the same 
c  name in GRFX03 (as of last changes made in July, 2015) that isolates 
c  all the calls to the PSPLOT library necessary for making a single 
c  page of possibly multiple frames, each with a single variable 
c  plotted.  There is no legend capability, and the NCAR option has been 
c  deleted.  While this version does not directly support writing a 
c  header (title) or footer to the graph, there is a standalone routine 
c  in this package called HDRFTR that can put a header and/or footer to 
c  the graph is desired.  For the full capabilities of GRFX03, that 
c  library should be used.
c
c  For clarity, the terms "page", "frame", and "plot" are used 
c  consistently.  Thus, on a single page, there may be numerous frames 
c  (np1h x np1v), and on each frame, there may be numerous plots (though 
c  in this version, only one plot per frame).  The word "graph" is used 
c  generically and refers to any graphical unit, be it a single plot, a 
c  frame, or the entire page.  Its meaning is gleaned from context.
c
c  If a plot is to start from within the frame rather than from the 
c  border (e.g., there may be no solution to what is being plotted for 
c  some values of the abscissa), the function value should be set to 
c  -1.1e-32 or more negative at the points to be ignored.  Such points 
c  will not have line segments drawn to or from them, even to intersect 
c  the border.
c
c  INPUT VARIABLES:
c    xvar        vector    of independent variable (x-axis)
c    yvar        vector(s) of   dependent variable (y-axis)
c    yexp        log10 of scaling factor generated when "yvar" was
c                converted to single precision (integer)
c    ibeg        index of first value in "xvar" and "yvar" to plot
c    iend        index of last  value in "xvar" and "yvar" to plot
c    linlog      = 1 => linear x and y;    = 2 => linear x, log y
c                = 3 => log x, linear y;   = 4 => log x and y
c    imm         = 0 => use input "ymn" and "ymx" to limit range of data
c                       plotted
c                =-1 => same as 0, and extend y-axis by one minor tick
c                = 1 => plot entire range of data; extend y-axis
c    xmn         minimum value on the x-axis
c    xmx         maximum value on the x-axis
c    ymn         minimum value on the y-axis
c    ymx         maximum value on the y-axis
c    corl        =-6 => open diamonds; one symbol per datum
c                =-5 => open pentagrams
c                =-4 => asterisk (*)
c                =-3 => open squares
c                =-2 => open triangles
c                =-1 => plus signs (+)
c                = 0 => crosses (x)
c                = 1 => open circles (o)
c                = 2 => smooth curve drawn through data
c    aspect      desired aspect ratio of axes.
c                < 1.0 => short, wide frame
c                = 1.0 => square frame
c                > 1.0 => tall, narrow frame
c    allxl       =2 => all x labels/annotations are drawn (default)
c                =1 => only the x-label/annotations on the bottom row
c                      of frames on the page are drawn.
c                =0 => labels aren't even drawn on the bottom row.
c                if allxl < 0, same as abs(allxl) except last annotation
c                is not drawn if it falls at xxmax.
c    allyl       =2 => all y labels/annotations are drawn (default)
c                =1 => only the y-label/annotations on the left column
c                      of frames on the page are drawn.
c                =0 => labels aren't even drawn on the left column.
c                if allyl < 0, same as abs(allyl) except last annotation
c                is not drawn if it falls at yymax.
c    np1h        number of frames to be arranged horizontally per page
c    ip1h        in the current row   , this is the ip1h'th frame
c    np1v        number of frames to be arranged vertically   per page
c    ip1v        in the current column, this is the ip1v'th frame
c    xlab        label for x-axis
c    ylab        label for y-axis
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    GRFXAXES
c    PSCHARS
c    SETLW               )
c    SETCOLR             )
c    OCIRCLE , TIMES     )
c    PLUS    , TRIANGLE  )---- PSPLOT routines
c    SQUARE  , ASTERISK  )
c    STAR    , DIAMOND   )
c    DSHCRV  , PLOT      )
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       integer       ndim
       real          tiny    , huge    
       parameter     ( ndim = 9999
     1               , tiny = 1.0e-32, huge = 1.0e32 )
c
       character*(*) xlab    , ylab    
       logical       lmax0   , lastpt  
       integer       yexp    , ibeg    , iend    , linlog  , imm      
     1             , corl    , allxl   , allyl   , np1h    , ip1h    
     2             , np1v    , ip1v    , lnlg    , nzi     , i       
     4             , i1      , ip1     , ip2     , ip3     , eye     
     6             , ndash   , n       , ista    , ifin    
       real          xmn     , xmx     , ymn     , ymx     , penwt   
     1             , aspect  , xmin    , xmax    , ymin    , ymax    
     2             , yming   , ymaxg   , yminl   , ymaxl   , aymaxl  
     2             , q1      , q2      , ymingt0 , p1x0    , p1y0       
     4             , size    , deks    , eksip1  , eksmin  , eksmax  
     7             , delwhy  , dwhyll  , dwhyl   , dwhyr   , dwhy    
     8             , dwhyrr  , whymin  , whymax  , whyip1  , ffx     
     9             , ffy     
c
       real          xvar    (   *), yvar    (   *)
     1             , eks     (ndim), ex      (ndim), why     (ndim)
c
       real          wxmin   , wxmax   , wymin   , wymax   , xxmin
     1             , xxmax   , yymin   , yymax   , scale
       common      / win1d /
     1               wxmin   , wxmax   , wymin   , wymax   , xxmin
     2             , xxmax   , yymin   , yymax   , scale
c
c      External statements
c
       external      grfxaxes, pschars , setlw   , setcolr , ocircle 
     1             , times   , plus    , triangle, square  , asterisk
     3             , star    , diamond , dshcrv  , plot     
c
c-----------------------------------------------------------------------
c
       lnlg  = max ( 1, min ( 4, linlog ) )
       penwt = 1.0
       call setlw ( 0.03 * penwt )
c
c      Set origin and symbol sizes.
c
       p1x0 = 4.5
       p1y0 = 10.8
       if (np1h*np1v .eq. 1) p1y0 = 13.4
       size = 0.085
c
c-----------------------------------------------------------------------
c---------------------------- E X T R E M A ----------------------------
c-----------------------------------------------------------------------
c
c      Set the domain, "xmin" and "xmax".  
c
       xmin = xmn
       xmax = xmx
       if (xmn .ge. xmx) then
         xmin = xvar(ibeg)
         xmax = xvar(iend)
       endif
c
c      If a log x-axis is desired (lnlg=3,4), make sure xvar > 0.  If
c  not, reset lnlg to give a linear x-axis and issue a warning.
c
       if (lnlg .ge. 3) then
         if (xmin .le. 0.0) then
           lnlg = lnlg - 2
           write (6, 2010)
         endif
       endif
c
c      Determine the actual x-limits of the plot ("ista" and "ifin").
c
       ista = ibeg
       ifin = iend
       do 20 i=ibeg,iend
         if ( xvar(i) .lt. xmin ) ista = i + 1
         if ( xvar(i) .le. xmax ) ifin = i
20     continue
c
c      Set the range of the frame.  Four measures of the extrema are 
c  considered:
c
c  1.  ymn, ymx    user-input extrema to be used for "yvar", and take
c                  precedent over all other evaluations.
c  2.  ymaxg,      global (ibeg:iend) extrema for "yvar", used below for
c        yming     detecting odd poles.
c  3.  ymaxl,      local (ista:iend) extrema for "yvar", used to compute
c        yminl     ymin and ymax.
c  4.  ymin, ymax  set to ymn, ymx if imm .le. 0, or the extrema of 
c                  ymaxl and yminl if imm = 1; used by GRFXAXES to 
c                  determine the scale for the ordinate.
c
       ymaxg =-huge
       yming = huge
       do 30 i=ibeg,iend
         ymaxg = max ( ymaxg, yvar(i) )
         yming = min ( yming, yvar(i) )
30     continue
       ymaxl =-huge
       yminl = huge
       do 40 i=ista,ifin
         if (yvar(i) .lt. huge) ymaxl = max ( ymaxl, yvar(i) )
         if (yvar(i) .gt.-huge) yminl = min ( yminl, yvar(i) )
40     continue
       aymaxl = max ( abs(yminl), abs(ymaxl) )
c
c      Set the range, "ymin" and "ymax".  
c
       if ( (imm .le. 0) .and. (ymx .gt. ymn) ) then
         ymin = ymn
         ymax = ymx
       else
         ymax = max (-huge, ymaxl )
         ymin = min ( huge, yminl )
       endif
c
c      Adjust extrema of range if both are zero, or if they differ by 
c  less than one part in 10**5.  NB: SIGN function is used (instead of 
c  testing ymin*ymax directly) in case ymin*ymax is larger than 1.0e38, 
c  thus triggering an overflow NaN.
c
       if ( (ymin .eq. 0.0) .and. (ymax .eq. 0.0) ) then
         ymin =-1.0
         ymax = 1.0
       else
         q1 = sign ( 1.0, ymin ) * sign ( 1.0, ymax )
         if (q1 .ge. 0.0) then
           q2 = 2.0 * ( ymax - ymin ) / ( ymax + ymin )
           if (abs(q2) .le. 1.0e-5) then
             if (ymax .gt. 0.0) then
               ymax = 1.005 * ymax
               ymin = 0.995 * ymin
             else
               ymax = 0.995 * ymax
               ymin = 1.005 * ymin
             endif
           endif
         endif
       endif
c
c      If a log y-axis is desired (lnlg=2,4), make sure yvar > 0.  If
c  ymin < 0 or if ymin = ymax = 0, reset "lnlg" to give a linear y-axis 
c  and issue a warning.  If ymin = 0 and ymax > 0, reset "ymin" and all 
c  zero-values of "yvar" (if any) to the smallest non-zero value in 
c  "yvar" ("ymingt0"), leave "lnlg" alone, and issue a warning.
c
       if ( (lnlg .eq. 2) .or. (lnlg .eq. 4) ) then
c
         if (ymin .lt. 0.0) then
           lnlg = lnlg - 1
           write (6, 2020)
         endif
c
         if (ymin .eq. 0.0) then
           lmax0 = .false.
           if (aymaxl .eq. 0.0) lmax0 = .true.
           if (lmax0) then
             lnlg = lnlg - 1
             write (6, 2021)
           else
             ymingt0 = huge
             do 90 i=ista,ifin
               if (yvar(i) .gt. 0.0) ymingt0 = min ( ymingt0, yvar(i) )
90           continue
             do 110 i=ista,ifin
               yvar(i) = max ( ymingt0, yvar(i) )
110          continue
             ymin = ymingt0
             write (6, 2030)
           endif
         endif
c
       endif
c
c-----------------------------------------------------------------------
c------------------------------- A X E S -------------------------------
c-----------------------------------------------------------------------
c
       call grfxaxes ( yexp, xlab, ylab, imm, xmin, xmax, ymin, ymax
     1               , aspect, p1x0, p1y0, allxl, allyl, np1h, ip1h
     2               , np1v, ip1v, lnlg )
c
c      Apply "scale" computed in GRFXAXES to font and symbol sizes.
c
       size = size * scale
c
c-----------------------------------------------------------------------
c-------------------------- P L O T   D A T A --------------------------
c-----------------------------------------------------------------------
c
       nzi = ifin - ista + 1
c
       call plot ( wxmin, wymin, 3 )
c
c      Set scaling factors "ffx" and "ffy".
c
       ffx = ( wxmax - wxmin ) / ( xxmax - xxmin )
       ffy = ( wymax - wymin ) / ( yymax - yymin )
       if (lnlg .ge. 3)
     1   ffx = ( wxmax - wxmin ) / log10 ( xxmax / xxmin )
       if ( (lnlg .eq. 2) .or. (lnlg .eq. 4) )
     1   ffy = ( wymax - wymin ) / log10 ( yymax / yymin )
c
c      Bounds to keep plots inside frame windows.
c
       eksmin = wxmin
       eksmax = wxmin + ffx * ( xxmax - xxmin )
       whymin = wymin
       whymax = wymin + ffy * ( yymax - yymin )
       if (lnlg .ge. 3)
     1   eksmax = wxmin + ffx * log10 ( xxmax / xxmin )
       if ( (lnlg .eq. 2) .or. (lnlg .eq. 4) )
     1   whymax = wymin + ffy * log10 ( yymax / yymin )
c
c       Fill array "eks" with scaled independent variable.  "ex" is a 
c  permanent copy of the abscissa should "eks" have to be adjusted to 
c  avoid a pole, start the graph on a border, etc.
c 
       do 220 i=1,nzi
         eye    = i + ista - 1
         eks(i) = wxmin + ffx * ( xvar(eye) - xxmin )
         if ( (lnlg .eq. 3) .or. (lnlg .eq. 4) )
     1     eks(i) = wxmin + ffx * log10 ( xvar(eye) / xxmin )
         ex (i) = eks(i)
220    continue
c
c       Fill array "why" with scaled dependent variable(s), and reset 
c  the abscissa.
c 
       do 250 i=1,nzi
         eye    = i + ista - 1
         why(i) = yvar(eye)
         if (why(i) .gt.-huge) then
           why(i) = wymin + ffy * ( yvar(eye) - yymin )
           if ( (lnlg .eq. 2) .or. (lnlg .eq. 4) )
     1       why(i) = wymin + ffy * log10 ( yvar(eye) / yymin )
         endif
         eks(i) = ex(i)
250    continue
c
c      Set colour.
c
       call setcolr ( 0.2, 0.2, 0.2 )
c
c      Draw plots with one symbol per datum, wikth the symbol type set 
c  by "corl".  Only points between "xmin" and "xmax" (by virtue of 
c  running "i" from "ista" to "ifin") and between "ymin" and "ymax" are 
c  plotted.
c
       if ( (corl .ge.-6) .and. (corl .le. 1) ) then
         do 260 i=1,nzi
           if ( ( eks(i) .le. eksmax ) .and. 
     1          ( eks(i) .ge. eksmin ) .and. 
     2          ( why(i) .le. whymax ) .and. 
     3          ( why(i) .ge. whymin ) ) then
             if (corl .eq. 1) call ocircle  ( eks(i), why(i), size )
             if (corl .eq. 0) call times    ( eks(i), why(i), size )
             if (corl .eq.-1) call plus     ( eks(i), why(i), size )
             if (corl .eq.-2) call triangle ( eks(i), why(i), size )
             if (corl .eq.-3) call square   ( eks(i), why(i), size )
             if (corl .eq.-4) call asterisk ( eks(i), why(i), size )
             if (corl .eq.-5) call star     ( eks(i), why(i), size )
             if (corl .eq.-6) call diamond  ( eks(i), why(i), size )
           endif
260      continue
       endif
c
c      Variables are plotted with lines.  Set line weight and dash 
c  pattern.
c
       if (corl .eq. 2) then
         call setlw ( 0.03 * penwt )
         ndash = 0
c
c      Scan the independent variable to make sure it stays within the
c  limits "whymax" and "whymin".  An effort is made to avoid drawing 
c  curves through "odd poles" (where the functional value goes from +/- 
c  infinity to -/+ infinity).
c
c  "i1"     is the beginning index of the next line segment to be drawn.
c  "delwhy" is a measure of the greatest jump in why(i) that might be 
c           expected without it being an odd pole.  There is no need to 
c           prepare "delwhy" for a possible log plot as an odd pole 
c           would already have reset the log-y request to a linear-y.
c  "lastpt" keeps track of whether why(ip1) is within the frame and is
c           used to determine if a line segment should be drawn.
c  
         i1     = 1
         delwhy = ( min ( ymax, ymaxg )
     1            - max ( ymin, yming ) ) * ffy * 0.8
         dwhyll = 0.0
         dwhyl  = 0.0
         dwhy   = why(i1+1) - why(i1  )
         dwhyr  = why(i1+2) - why(i1+1)
         lastpt = .false.
         if ( (why(i1) .ge. whymin) .and. 
     1        (why(i1) .le. whymax) ) lastpt = .true.
         do 270 i=1,nzi-1
           ip1    = i + 1
           ip2    = min ( i+2, nzi )
           ip3    = min ( i+3, nzi )
           dwhyrr = why(ip3) - why(ip2)
           deks   = eks(ip1) - eks(i  )
c
c      Note from DAC, 7/15: last two criteria added to help prevent 
c  points flagged not to be plotted (by setting why=-huge in calling 
c  routine) from being mistaken for an odd pole.  Presumably, "why" 
c  never gets to -huge, even for an odd pole.
c
           if ( ( dwhyll*dwhyl    .gt. 0.0    ) .and.
     1          ( dwhyl *dwhy     .lt. 0.0    ) .and. 
     2          ( dwhy  *dwhyr    .lt. 0.0    ) .and.
     3          ( dwhyr *dwhyrr   .gt. 0.0    ) .and.
     3          ( why(i)*why(ip1) .lt. 0.0    ) .and.
     4          ( abs(dwhy)       .gt. delwhy ) .and.
     3          ( why(i)          .gt.-huge   ) .and.
     1          ( why(ip1)        .gt.-huge   ) ) then
c
c      An odd pole has been detected.  If a line segment is pending
c  (e.g., lastpt .eq. .true.), draw the current segment up to the left
c  of the pole, and set the starting point for the next line segment.
c
             if ( lastpt ) then
               q1 = deks / ( dwhy + tiny )
               if ( why(i) .gt. why(ip1) ) then
                 eks(i  ) = eks(i) + ( whymax - why(i) ) * q1 
                 why(i  ) = whymax
                 eks(ip1) = eks(i) + ( whymin - why(i) ) * q1
                 why(ip1) = whymin
               else
                 eks(i  ) = eks(i) + ( whymin - why(i) ) * q1 
                 why(i  ) = whymin
                 eks(ip1) = eks(i) + ( whymax - why(i) ) * q1 
                 why(ip1) = whymax
               endif
               call dshcrv ( eks(i1), why(i1), ip1-i1, ndash, 0.0 )
               i1 = ip1
             endif
c
           else
c
c      If why(i) and why(ip1) do not straddle an odd pole, perform the 
c  tests necessary to keep the plot inside the frame window.
c
c  1.  If "why" enters the frame from below, interpolate eks(i) to the 
c      entry point, and reset the initial point of the current segment.
c
c  Note from DAC, 7/15: the if-test on -huge here and in case 3 below is
c  the mechanism introduced to allow some points not to be plotted at 
c  all.  Such points are set to -1.0e32 in the calling routine, and are 
c  then skipped altogether (as opposed to being interpolated to the plot 
c  border for points outside the frame).
c
             if ( (why(i  ) .lt. whymin) .and. 
     1            (why(ip1) .gt. whymin) ) then
               eks(i) = eks(i) + ( whymin - why(i) ) * deks / dwhy 
               if (why(i) .le.-huge) then
                 why(i) = why(ip1)
               else
                 why(i) = whymin
               endif
               i1 = i
             endif
c
c  2.  If "why" enters the frame from above, interpolate eks(i) to the 
c      entry point, and reset the initial point of the current segment.
c
             if ( (why(i  ) .gt. whymax) .and. 
     1            (why(ip1) .lt. whymax) ) then
               eks(i) = eks(i) + ( whymax - why(i) ) * deks / dwhy 
               why(i) = whymax
               i1     = i
             endif
c
c  3.  If "why" falls below the frame, interpolate eks(ip1) to the exit 
c      point, draw the current segment to the exit point, and set the 
c      starting point for the next segment.
c
             if ( (why(i  ) .gt. whymin) .and. 
     1            (why(ip1) .lt. whymin) ) then
               eksip1   = eks(ip1)
               whyip1   = why(ip1)
               eks(ip1) = eks(i) + ( whymin - why(i) ) * deks / dwhy 
               if (why(ip1) .le.-huge) then
                 why(ip1) = why(i)
               else
                 why(ip1) = whymin
               endif
               call dshcrv ( eks(i1), why(i1), i-i1+2, ndash, 0.0 )
               eks(ip1) = eksip1
               why(ip1) = whyip1
               i1       = ip1
             endif
c
c  4.  If "why" rises above the frame, interpolate eks(ip1) to the exit 
c      point, draw the current segment to the exit point, and set the 
c      starting point for the next segment.
c
             if ( (why(i  ) .lt. whymax) .and. 
     1            (why(ip1) .gt. whymax) ) then
               eksip1   = eks(ip1)
               whyip1   = why(ip1)
               eks(ip1) = eks(i) + ( whymax - why(i) ) * deks / dwhy 
               why(ip1) = whymax
               call dshcrv ( eks(i1), why(i1), i-i1+2, ndash, 0.0 )
               eks(ip1) = eksip1
               why(ip1) = whyip1
               i1       = ip1
             endif
c
           endif
c
c      Update "lastpt".
c
           lastpt = .false.
           if ( (why(ip1) .ge. whymin) .and. 
     1          (why(ip1) .le. whymax) ) lastpt = .true.
c
c      Advance "dwhyll" to "dwhyl", etc., before next iteration.
c
           dwhyll = dwhyl
           dwhyl  = dwhy
           dwhy   = dwhyr
           dwhyr  = dwhyrr
270      continue
c
c      Draw last segment, if necessary.
c
         if ( lastpt ) then
           call dshcrv ( eks(i1), why(i1), nzi-i1+1, ndash, 0.0 )
         endif
c
       endif
c
c      Restore pen weight and colour before completing graph.
c
       call setlw   ( 0.03 * penwt )
       call setcolr ( 0.0, 0.0, 0.0 )
c
c-----------------------------------------------------------------------
c----------------------- Write format statements -----------------------
c-----------------------------------------------------------------------
c
2010   format('GRFX1D  : |---WARNING---> The x-axis includes the '
     1       ,'origin.  Log option switched',/
     2       ,'GRFX1D  : to linear.')
2020   format('GRFX1D  : |---WARNING---> At least one dependent '
     1       ,'variable includes negative',/
     2       ,'GRFX1D  : values.  Log option switched to linear.')
2021   format('GRFX1D  : |---WARNING---> At least one dependent '
     1       ,'variable includes no positive',/
     2       ,'GRFX1D  : values.  Log option switched to linear.')
2030   format('GRFX1D  : |---WARNING---> The y-axis includes the '
     1       ,'origin.  Zeroes reset to ',/
     2       ,'GRFX1D  : minimum non-zero value; log option retained.')
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine grfxcls
c
c    dac:grfx03.grfxcls <---------------------------- closes a plot file
c                                                             june, 1990
c
c    written by: David Clarke
c    modified 1: 
c
c  PURPOSE: This subroutine closes a PSPLOT plot file and should be 
c  called after each call to GRFX1D and before the next plot is begun.
c
c  INPUT VARIABLES:
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    PLOT     )---- PSPLOT routine
c
c-----------------------------------------------------------------------
c
       implicit      none
c
c      External statements
c
       external      plot
c
c-----------------------------------------------------------------------
c
       call plot ( 0.0, 0.0, 999 )
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine grfxopn ( fname )
c
c    dac:grfx03.grfxopn <----------------------------- opens a plot file
c                                                             june, 1990
c
c    written by: David Clarke
c    modified 1:
c
c  PURPOSE: This subroutine opens a PSPLOT plot file with the specified 
c  file name.  It should be called before GRFX1D is called.
c
c  INPUT VARIABLES:
c    fname       file name of plot file as set by calling routine
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    NEWDEV  )---- PSPLOT routines
c    PSINIT  )
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) fname   
c
c      External statements
c
       external      newdev  , psinit
c
c-----------------------------------------------------------------------
c
       call newdev ( fname )
       call psinit ( .true. )
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine hdrftr ( phdr, iftr, filename )
c
c    dac:1dplot.hdrftr <------------------ writes header/footer to graph
c                                                             june, 2018
c
c    written by: David Clarke
c    modified 1: 
c
c  PURPOSE: This subroutine writes the header (title) "phdr" to the top-
c  centre of the graph.  If desired, a footer with the filename and the 
c  date and time of creation is created and written at the bottom left 
c  of the graph.
c
c  INPUT VARIABLES:
c    phdr        header (title) to be written to graph.
c                blank => no header
c    iftr        =1 => footer created and written to graph
c                =0 => not
c    filename    used to create footer
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    SETLW            )  PSPLOT routine
c    STRLEN
c    PSCHARS
c    TIMEDATE
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) phdr    , filename
       character*8   chtime
       character*10  chdate
       character*80  pftr
       integer       iftr    , nch   
       real          pt12    , pt20    , x1      , y1      
c
c      External statements
c
       integer       strlen
       external      setlw   , strlen  , pschars , timedate
c
c-----------------------------------------------------------------------
c
c      Set line weight and font sizes.
c
       call setlw ( 0.03 )
       pt12 = 0.264
       pt20 = 0.440
c
c      Write user-specified title, if desired.
c
       nch  = strlen ( phdr )
       if (nch .gt. 0) then
         x1 = 9.5
         y1 = 24.3
         call pschars ( x1, y1, pt20, phdr(1:nch), 0.0, 1 )
       endif
c
c      Create and write footer, if desired.
c
       if (iftr .eq. 1) then
         call timedate ( chtime, chdate )
         nch = strlen ( filename )
         write ( pftr, 2010 ) filename(1:nch), chdate, chtime
         nch = strlen ( pftr )
         x1  = 2.5 
         y1  = 7.0
         call pschars ( x1, y1, pt12, pftr(1:nch), 0.0, 0 )
       endif
c
c-----------------------------------------------------------------------
c
2010   format('Plot file `',a,''' created on ',a10,' at ',a8,'.')
c
       return
       end
c
c=======================================================================
c
c>>>>>>>>>>>>>>>>>>>>> INTERNALLY CALLED ROUTINES <<<<<<<<<<<<<<<<<<<<<<
c
c++++++++++++++++++++++++ GRFX03 (DAC) ROUTINES ++++++++++++++++++++++++
c
c=======================================================================
c
       subroutine axislab ( xory, wxmin, wxmax, wymin, wymax, scale
     1                    , wconv, nchmax, clab, fact, iexp )
c
c    dac:grfx03.axislab <----------------------------------- labels axis
c                                                             june, 1996
c
c    written by: David Clarke
c    modified 1: December 1996, by David Clarke; added multiple plot per
c                frame capability.
c    modified 2: June, 2003, by David Clarke; generalised for use by 2-D
c                plotter as well.
c    modified 3: October 2003 by Alexander Men'shchikov; generalized to
c                include the PSPLOT graphics library.
c    modified 5: June, 2014 by DAC; introduced "fact" to scale down 
c                annotation sizes for long, short plots.
c
c  PURPOSE: This subroutine labels the axis.
c
c  INPUT VARIABLES:
c    xory        =1 => label the x-axis
c                =2 => label the y-axis
c    wxmin       minimum x-window coordinate
c    wxmax       maximum x-window coordinate
c    wymin       minimum y-window coordinate
c    wymax       maximum y-window coordinate
c    scale       scaling factor for multiple plots per frame
c    wconv       window conversion factor.  1-D plots are done in window
c                coordinates, and thus wconv = 1.0.  2-D plots are done
c                in grid coordinates, and thus wconv = width of window
c                in grid units.
c    nchmax      maximum number of characters in y-annotation (needed
c                so the y-axis label won't overwrite the annotations).
c    clab        character string containing the label.  Should be
c                written in `NCAR markup language'.
c    fact        scale factor of annotation size for long, short plots
c    iexp        .eq. 0 => ignored
c                .ne. 0 => "iexp" is the exponent of the annotations
c                          expressed in scientific notation and is
c                          incorporated into the axis label.
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    STRLEN
c    PSCHARS
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) clab
       character*34  clab2
       integer       xory    , iexp    , nchmax  , nch     , nch2
     1             , aexp
       real          xpos    , ypos    , angle   , wxmin   , wxmax
     1             , wymin   , wymax   , scale   , wlab    , wlab2
     2             , fnt1    , fnt2    , wconv   , q1      , q2
     3             , fact    , scfa    , scfa2
c
c      External statements
c
       integer       strlen
       external      strlen  , pschars
c
c-----------------------------------------------------------------------
c
       scfa  = scale * fact
       scfa2 = scale * fact**0.8
       fnt1  = 0.5 * scfa
       fnt2  = 0.4 * scfa
       q1    = wconv * scfa
       q2    = 0.02  * q1
c
c      Determine the number of characters in "clab".
c
       nch  = strlen ( clab )
       if (nch .eq. 0) return
c
c      If necessary, create a label for the axis scaling factor.
c
       wlab2 = 0.0
       if (iexp .ne. 0) then
         iexp = min ( 999, max ( -999, iexp ) )
         nch2 = 17
         clab2(1:nch2) = ' (|F18|U|F0|10|S|'
         if (iexp .lt. 0) then
           clab2(nch2+1:nch2+10) = '|F18|-|F0|'
           nch2 = nch2 + 10
         endif
         aexp = abs ( iexp )
         if (aexp .lt. 10) then
           write ( clab2(nch2+1:nch2+1), 2010 ) aexp
           nch2 = nch2 + 1
         endif
         if ( ( (aexp .ge. 10) .and. (aexp .lt. 99) ) ) then
           write ( clab2(nch2+1:nch2+2), 2020 ) aexp
           nch2 = nch2 + 2
         endif
         if (aexp .ge. 100) then
           write ( clab2(nch2+1:nch2+3), 2030 ) aexp
           nch2 = nch2 + 3
         endif
         clab2(nch2+1:nch2+4) = '|N|)'
         nch2 = nch2 + 4
       endif
c
c      Set coordinates and orientation of label, depending on whether
c  this is the x- or y-axis, and depending on whether the exponent of
c  the annotations has to be incorporated into the axis label.
c
       if (iexp .ne. 0.0) wlab2 = 4.5 * fnt2
       if (xory .eq. 1) then
         angle = 0.0
         xpos  = 0.5 * ( wxmax + wxmin - wlab2 ) + 0.1 * scfa
         ypos  = wymin - 1.6 * scfa2
       else
         angle = 90.0
         xpos  = wxmin - 0.75 * fnt1 * real ( nchmax ) - 0.15 * scale
         ypos  = 0.5 * ( wymax + wymin - wlab2 ) + 0.1 * scfa
       endif
c
c      Write the axis label.
c
       call pschars ( xpos, ypos, fnt1, clab(1:nch), angle, 1 )
       if (iexp .ne. 0) then
         call pschars ( 999.0, 999.0, fnt2, clab2(1:nch2), angle, 1)
       endif
c
c-----------------------------------------------------------------------
c----------------------- Write format statements -----------------------
c-----------------------------------------------------------------------
c
2010   format ( i1 )
2020   format ( i2 )
2030   format ( i3 )
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine axisset1 ( xory, qmin, qmax, imm, aspect, ndec, iexp
     1                     , nchq, qqmin, qqmax, nqdiv, nqann, qann
     2                     , cqann)
c
c    dac:grfx03.axisset1 <------------------- parameters for linear axis
c                                                             june, 1996
c
c    written by: David Clarke
c    modified 2: July, 2010 by DAC; modified to give proper minus signs 
c                for PSPLOT, and upgraded over-crowding algorithm to 
c                take into account number of characters per annotation.
c
c  PURPOSE: This subroutine computes the parameters needed to draw a
c  linear axis.  Parameters for logarithmic axes are computed by
c  AXISSET2.  Constraints include:
c
c      1. annotation and tick spacings chosen so they are not crowded.
c      2. annotations will always be multiples of 1, 2, or 5.
c      3. annotations will be expressed in scientific notation if the
c         absolute value of the annotation is .le. 0.001 or .ge. 1000.
c      4. new extrema are chosen to be multiples of the tick spacing.
c      5. Axes are not extended to fall exactly on an annotated tick.
c      6. y-axes are extended by one minor tick for imm=1,-1.
c
c  INPUT VARIABLES:
c    xory        = 1 => x-axis
c                = 2 => y-axis
c                = 3 => polar axis
c    qmin        minimum value on axis.
c    qmax        maximum value on axis.
c    imm         = 1,-1 =>     extend grid by one minor tick mark
c                = 0 => do not extend grid by one minor tick mark
c    aspect      desired aspect ratio of axes.
c                < 1 => short, wide plot
c                = 1 => square plot
c                > 1 => tall, narrow plot
c    ndec        the number of significant figures to appear to the
c                right of the decimal in the annotations.
c                =-1 => minimum number of decimal places required to
c                       make each annotation unique.
c
c  OUTPUT VARIABLES:
c    iexp        If the absolute annotation values are .le. 0.001 or
c                .ge. 1000., "iexp" is the exponent of the annotations
c                expressed in scientific notation.  Otherwise, iexp = 0.
c    nchq        maximum number of characters in any annotation.
c    qqmin       greatest multiple of the major tick spacing < "qmin".
c    qqmax       smallest multiple of the major tick spacing > "qmax".
c    nqdiv       total number of divisions along axis (always > 0).
c    nqann       total number of annotations along axis.
c    qann        1-D real array containing the annotation locations
c                along the axis.
c    cqann       1-D character array (at least character*21) containing 
c                the axis annotations in NCAR markup language.
c
c  LOCAL VARIABLES:
c    ccqann      temporary 1-D character array.
c    mant        1-D integer array containing int(qann).
c    idec        1-D integer array containing the decimal portion of
c                "qann".
c
c  EXTERNALS: [NONE]
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       integer       mann
       parameter     ( mann=20 )
c
       integer       xory    , ndec    , iexp    , nchq    , nqdiv
     1             , nqann   , i1      , i       , j       , jj
     2             , nmax    , imm     , ndeco   , nchars
       real          qmin    , qmax    , aspect  , qqmin   , qqmax
     1             , fact    , dq1     , dq2     , dq3     , dq4
     2             , dq5     , dq6     , qamin   , qamax   , sqmin
     3             , sqmax   , sqqmin  , sqqmax  , aqamax  , q1
     4             , q2      , qdec    , tick
c
       character*(*) cqann   (   *)
       character*16  ccqann  (mann)
       integer       mant    (mann), idec    (mann)
       real          qann    (   *)
c
c-----------------------------------------------------------------------
c
       ndeco = ndec
       if (xory .eq. 1) fact = 0.1 * max ( 1.0, aspect )
       if (xory .eq. 2) fact = 0.1 / min ( 1.0, aspect )
       if (xory .eq. 3) fact = 0.1 * max ( 1.0, aspect )
c
c      dq1    is the tentative spacing between annotations.
c      i1     is chosen so that 0.1 .lt. dq2 = dq1 * 10**(-i1) .le. 1.0
c      dq3    is the spacing between annotations (modulo a factor
c             10**i1)
c
       dq1   = ( qmax - qmin ) * fact
       i1    = int ( alog10 ( dq1 ) )
       if (dq1 .gt. 1.0) i1 = i1 + 1
       dq2   = dq1 * 10.0**(-i1)
       if (dq2 .le. 0.1) then
         i1  = i1 - 1
         dq2 = dq2 * 10.0
       endif
       dq3   = 1.0
       if ( (dq2 .gt. 0.2) .and. (dq2 .le. 0.5) ) dq3 = 0.5
       if ( (dq2 .gt. 0.1) .and. (dq2 .le. 0.2) ) dq3 = 0.2
c
c      dq4    is the final spacing between annotations (dq3 * 10**i1)
c      dq5    is the final spacing between ticks
c      qqmin  is the greatest multiple of dq5 less    than  qmin
c      qamin  is the smallest multiple of dq4 greater than qqmin
c      qqmax  is the smallest multiple of dq5 greater than  qmax
c      qamax  is the greatest multiple of dq4 less    than qqmax
c
10     continue
       dq4    = dq3 * 10.0**i1
       tick   = 5.0
       if ( dq3 .eq. 0.2 ) tick = 4.0
       dq5    = dq4 / tick
c
       sqmin  = sign ( 0.5 , qmin )
       sqmax  = sign ( 0.5 , qmax )
       qqmin  = real ( int ( qmin / dq5 - 0.499 + sqmin ) ) * dq5
       qqmax  = real ( int ( qmax / dq5 + 0.499 + sqmax ) ) * dq5
       if ( (xory .eq. 2) .and. (imm .ne. 0) ) then
         if ( qmin-qqmin .lt. 0.5*dq5) qqmin = qqmin - dq5
         if (qqmax- qmax .lt. 0.5*dq5) qqmax = qqmax + dq5
       endif
       if (xory .eq. 3) then
         if ( (qqmin .gt.  0.0) .and. (qqmin .lt.  5.0) ) qqmin =  0.0
         if ( (qqmax .gt. 85.0) .and. (qqmax .lt. 90.0) ) qqmax = 90.0
       endif
       sqqmin = sign ( 0.5, qqmin )
       sqqmax = sign ( 0.5, qqmax )
       qamin  = real ( int ( qqmin / dq4 + 0.499 + sqqmin ) ) * dq4
       qamax  = real ( int ( qqmax / dq4 - 0.499 + sqqmax ) ) * dq4
c
c      nqann  is the number of annotations along the axis
c      nqdiv  is the number of divisions along the axis (always > 0)
c
c      Note: if nqann > nmax, the value of dq3 is increased and
c  execution is returned to statement 10 above so that nqann is .le.
c  nmax.
c
       nqann = int ( ( qamax - qamin ) / dq4 + 0.01 ) + 1
       nmax  = int ( 8.0 / max ( 1.0, aspect ) + 0.5 )
       if (xory .eq. 2)
     1   nmax = int ( 8.0 * min ( 1.0, aspect ) + 0.5 )
       if (nqann .gt. nmax) then
         if (dq3 .eq. 0.2) then
           dq3 = 0.5
           go to 10
         endif
         if (dq3 .eq. 0.5) then
           dq3 = 1.0
           go to 10
         endif
         if (dq3 .eq. 1.0) then
           dq3 = 0.2
           i1 = i1 + 1
           go to 10
         endif
       endif
       nqdiv = int ( ( qqmax - qqmin ) / dq5 + 0.01 )
c
c      If the largest absolute value at a major tick mark (aqmax) is
c  greater than 1000 or less than 0.01, the axis values are printed in
c  scientific notation where the mantissa is written next to the major
c  tick mark and the exponent (q1 = 10**iexp) is given as part of the
c  axis label.
c      q2     is the reciprocal of q1.
c      dq6    is the spacing between annotations, modulo q2
c
       aqamax = max ( abs(qamax), abs(qamin) )
       iexp  = 0
       if (aqamax .ge. 1000.) iexp = int ( alog10 ( aqamax ) )
       if (aqamax .lt. 0.01 ) iexp = int ( alog10 ( 0.1001 * aqamax ) )
       q1    = 10.0**iexp
       q2    = 10.0**(-iexp)
       dq6   = dq4 * q2
c
c      ndec   is the number of decimal places to appear in the mantissa.
c
c      If ndec=-1, then the routine automatically selects ndec to the
c  minimum number required for all annotations along the axis to be
c  different.  e.g., for the values 1.210, 1.220, 1.230, 1.240,
c  1.250, ndec should be 2.  If it were 1, then all values would appear
c  as 1.2.
c      If the automatic routine doesn't work for some pathelogical case,
c  or if the user requires additional zeroes after the last decimal
c  place chosen by the routine, then specifying ndec will override the
c  automatic routine.  Note that ndec can be no larger than 6.
c
       if (ndeco .eq. -1) ndec =-int ( alog10 ( dq6 ) - 0.7 )
       ndec = max ( 0, min ( 6, ndec ) )
       if ( (aqamax .lt. 20.0) .and. (ndec .eq. 0) ) ndec = 1
c
c      qann   is a real 1-D array with the annotations for the axis.
c
       qann(1) = qamin * q2
       do 20 i=2,nqann
         qann(i) = qann(i-1) + dq6
20     continue
c
c      The annotations are converted to character strings with all the
c  machine round-off noise cleaned out.  This is done by separating the
c  portions to the left (mant) and right (idec) of the decimal point,
c  then formatting the character string according to the value of
c  "ndec".  Loop 40 squeezes out extraneous blanks.
c
       do 30 i=1,nqann
         mant(i) = int ( abs ( qann(i) ) + 0.000005 )
         qdec    = abs ( qann(i) ) - real ( mant(i) )
         idec(i) = int ( 0.1 + 10.0**ndec * qdec )
         qann(i) = qann(i) * q1
30     continue
       nchq = 1
       do 50 i=1,nqann
         if (ndec .eq. 0) write (ccqann(i),2010) mant(i)
         if (ndec .eq. 1) write (ccqann(i),2020) mant(i), idec(i)
         if (ndec .eq. 2) write (ccqann(i),2030) mant(i), idec(i)
         if (ndec .eq. 3) write (ccqann(i),2040) mant(i), idec(i)
         if (ndec .eq. 4) write (ccqann(i),2050) mant(i), idec(i)
         if (ndec .eq. 5) write (ccqann(i),2060) mant(i), idec(i)
         if (ndec .eq. 6) write (ccqann(i),2070) mant(i), idec(i)
         cqann(i) = ' '
         jj = 0
         if (qann(i) .lt. 0.0) then
           cqann(i)(1:10) = '|F18|-|F0|'
           jj = 10
         endif
         do 40 j=1,16
           if (ccqann(i)(j:j) .ne. ' ') then
             jj = jj + 1
             cqann(i)(jj:jj) = ccqann(i)(j:j)
           endif
40       continue
         if (qann(i) .lt. 0.0) jj = jj - 9
         nchq = max ( nchq, jj )
50     continue
c
c      To avoid x-axis crowding, count the total number of characters 
c  (excluding decimal points) to be printed along the x-axis.  If this
c  is greater than 30, reset "dq3" and return to statement 10 to try
c  again.
c
       nchars = nqann * ( nchq - 1 )
       if ( (xory .eq. 1) .and. (nchars .gt. 30) ) then
         if (dq3 .eq. 0.2) then
           dq3 = 0.5
           go to 10
         endif
         if (dq3 .eq. 0.5) then
           dq3 = 1.0
           go to 10
         endif
         if (dq3 .eq. 1.0) then
           dq3 = 0.2
           i1  = i1 + 1
           go to 10
         endif
       endif
c
c      "nchq" is returned so that the y-axis label may be positioned 
c  just left of the left-most character in the annotations.
c
       nchq = nchq + 1
       if (ndec .eq. 0) nchq = nchq + 1
c
c-----------------------------------------------------------------------
c----------------------- Write format statements -----------------------
c-----------------------------------------------------------------------
c
2010   format(i4,12x)
2020   format(i4,'.',i1.1,10x)
2030   format(i4,'.',i2.2, 9x)
2040   format(i4,'.',i3.3, 8x)
2050   format(i4,'.',i4.4, 7x)
2060   format(i4,'.',i5.5, 6x)
2070   format(i4,'.',i6.6, 5x)
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine axisset2 ( xory, qmin, qmax, yexp, aspect, nchq, qqmin
     1                     , qqmax, nqdiv, nqann, qann, cqann )
c
c    dac:grfx03.axisset2 <-------------- parameters for logarithmic axis
c                                                             july, 1996
c
c    written by: David Clarke
c    modified 2: July, 2010 by DAC; modified to give proper minus signs 
c                for PSPLOT.
c
c  PURPOSE: This subroutine computes the parameters needed to draw a
c  logarithmic axis.  Parameters for linear axes are computed by
c  AXISSET1.  Constraints include:
c
c      1. annotation and tick spacings chosen so they are not crowded.
c      2. annotations will always be powers of 10.
c      3. annotations will be expressed in scientific notation if the
c         absolute value of the annotation is .le. 0.001 or .ge. 1000.
c      4. new extrema are chosen to be multiples of the tick spacing.
c      5. Axes are not extended to fall exactly on an annotated tick.
c
c  INPUT VARIABLES:
c    xory        = 1 => x-axis
c                = 2 => y-axis
c    qmin        minimum value on axis.
c    qmax        maximum value on axis.
c    yexp        log10 of scaling factor generated when y was converted
c                to single precision
c    aspect      desired aspect ratio of axes.
c                < 1 => short, wide plot
c                = 1 => square plot
c                > 1 => tall, narrow plot
c
c  OUTPUT VARIABLES:
c    nchq        maximum number of characters in any annotation.
c    qqmin       location of first tick mark.
c    qqmax       location of last  tick mark.
c    nqdiv       >0 => one less than the total number of decades along
c                      axis.  These will mark location of minor ticks.
c                =0 => minor ticks will mark units within each decade.
c    nqann       total number of annotations along axis.
c    qann        1-D real array containing the annotation locations
c                along the axis.
c    cqann       1-D character array (at least character*21) containing 
c                the axis annotations in NCAR markup language.
c
c  LOCAL VARIABLES:
c
c  EXTERNALS: [NONE]
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       integer       mann
       parameter     ( mann=20 )
c
       integer       xory    , yexp    , nchq    , nqdiv   , nqann   
     1             , i1      , i2      , i3      , i4      , i5      
     2             , i6      , i7      , i8      , dnq     , i
     3             , n
       real          qmin    , qmax    , aspect  , qqmin   , qqmax
     1             , fact    , ri5     , ri6     , rdnqi
c
       character*(*) cqann   (   *)
       real          qann    (   *)
c
c-----------------------------------------------------------------------
c
c      i1     is the highest power of ten less than "qmin".
c      i2     is a multiple of 10**i1 less than "qmin".
c      qqmin  is the log of the location of the first tick mark
c
       i1    = int ( alog10 ( qmin * 1.01 ) )
       if (qmin .lt. 1.0) i1 = i1 - 1
       i1    = max ( -32, min ( 32, i1 ) )
       i2    = int ( qmin * 10.0**(-i1) + 0.01 )
       if (i2 .ge. 8) i2 = i2 - 1
       if (i2 .ge. 6) i2 = i2 - 1
       if (i2 .ge. 4) i2 = i2 - 1
       qqmin = alog10 ( real ( i2 ) ) + real ( i1 )
c
c      i3     is the highest power of ten less than "qmax".
c      i4     is a multiple of 10**i3 greater than "qmax".
c      qqmax  is the log of the location of the last tick mark
c
       i3    = int ( alog10 ( qmax * 0.99 ) )
       if (qmax .lt. 1.0) i3 = i3 - 1
       i3    = max ( -32, min ( 32, i3 ) )
       i4    = int ( qmax * 10.0**(-i3) + 1.01 )
       if (i4 .ge. 8) i4 =            i4 + 1
       if (i4 .ge. 6) i4 = min ( 10, i4 + 1 )
       if (i4 .ge. 4) i4 = min ( 10, i4 + 1 )
       qqmax = alog10 ( real ( i4 ) ) + real ( i3 )
c
c      If "qqmin" and "qqmax" fall within the same decade, reset them to
c  the extremes of that decade.
c
       if (i3 .eq. i1) then
         qqmin = real ( i1     )
         qqmax = real ( i1 + 1 )
       endif
c
c      i5     is the smallest integer greater than "qqmin"
c      i6     is the largest  integer less    than "qqmax"
c
       i5    = int ( qqmin - 0.001 )
       if (qqmin .gt. 0.0) i5 = i5 + 1
       i6    = int ( qqmax + 0.001 )
       if (qqmax .lt. 0.0) i6 = i6 - 1
c
c      To avoid over-crowded annotations, only every "dnq"'th decade is
c  annotated.  If there are fewer than 8 or 9 decades between "qqmin"
c  and "qqmax", "dnq" is 1 and minor ticks mark off units within each
c  decade.  If there are more than 8 or 9 decades, "dnq" is greater than
c  1 and minor ticks mark off non-annotated decades.
c
c      i5     is the log of the location of the first annotation.
c      i6     is the log of the location of the last  annotation.
c
       if (xory .eq. 1) fact = max ( 1.0, aspect ) / 8.0
       if (xory .eq. 2) fact = 0.1 / min ( 1.0, aspect )
       dnq = int ( real ( i6 - i5 ) * fact ) + 1
       if (dnq .gt. 1) then
         ri5   = real ( i5 )
         ri6   = real ( i6 )
         rdnqi = 1.0 / real ( dnq )
         i5    = int ( ri5 * rdnqi + sign ( 0.001, ri5 ) ) * dnq
         i6    = int ( ri6 * rdnqi + sign ( 0.999, ri6 ) ) * dnq
         qqmin = real ( i5 )
         qqmax = real ( i6 )
         nqann = ( i6 - i5 ) / dnq + 1
c        nqdiv = i2 - i1
         nqdiv = i6 - i5
       else
         nqann = i6 - i5 + 1
         nqdiv = 0
       endif
c
c      Create real and character annotations for the axis.
c
       i5    = i5 + yexp
       i6    = i6 + yexp
       i7    = i5
       nchq  = 4
       do 10 i=1,nqann
         qann (i) = real ( i7 - yexp )
         cqann(i) = ' '
         i8 = abs ( i7 )
         if (i8 .le. 2) then
           if (i7 .eq. -2) then
             cqann(i)(1:4) = '0.01'
             nchq = max ( nchq, 5 )
           endif
           if (i7 .eq. -1) cqann(i)(1:3) = '0.1'
           if (i7 .eq.  0) cqann(i)(1:3) = '1.0'
           if (i7 .eq.  1) then
             cqann(i)(1:4) = '10.0'
             nchq = max ( nchq, 5 )
           endif
           if (i7 .eq.  2) then
             cqann(i)(1:5) = '100.0'
             nchq = max ( nchq, 6 )
           endif
         else
           cqann(i)(1:5) = '10|S|'
           n = 5
           if (i7 .lt. 0) then
             cqann(i)(6:15) = '|F18|-|F0|'
             n = 15
           endif
           if (i8 .le. 9) write ( cqann(i)(n+1:n+1), '(i1)' ) i8
           if ( (i8 .ge. 10) .and. (i8 .le. 99) )
     1       write ( cqann(i)(n+1:n+2), '(i2)' ) i8
           if (i8 .ge. 100) write ( cqann(i)(n+1:n+3), '(i3)' ) i8
         endif
         i7 = i7 + dnq
10     continue
       if ( (i5 .lt.    0) .or. (i6 .ge.  10) ) nchq = max ( nchq, 5 )
       if ( (i5 .le.  -10) .or. (i6 .ge. 100) ) nchq = 6
       if   (i5 .le. -100)                      nchq = 7
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine grfxaxes ( yexp, xlab, ylab, imm, xmin, xmax, ymin
     1                     , ymax, aspect, p1x0, p1y0, allxl, allyl
     2                     , np1h, ip1h, np1v, ip1v, lnlg )
c
c    dac:grfx03.grfxaxes <---------------------- draws axes for graphics
c                                                             june, 1996
c
c    written by: David Clarke
c    modified 1: December 1996, by David Clarke; added multiple frame 
c                per page capability.
c    modified 2: October 2003 by Alexander Men'shchikov; generalized to
c                include the PSPLOT graphics library.
c    modified 3: June, 2010 by DAC; moved determination of extrema and 
c                log-plot checks to GRFX1D.
c
c  PURPOSE: This subroutine draws and labels the axes (perimeter style)
c  for 1-D PSPLOT graphics.
c
c  INPUT VARIABLES:
c    yexp        log10 of scaling factor generated when y was converted
c                to single precision
c    xlab        character string for x-axis label (NCAR markup 
c                language).
c    ylab        character string for y-axis label (ditto).
c    imm         = 1,-1 =>     extend grid by one minor tick mark
c                = 0 => do not extend grid by one minor tick mark
c    xmin        minimum value on the x-axis
c    xmax        maximum value on the x-axis
c    ymin        minimum value on the y-axis
c    ymax        maximum value on the y-axis
c    aspect      desired aspect ratio of axes.
c                < 1 => short, wide frame
c                = 1 => square frame
c                > 1 => tall, narrow frame
c    p1x0        x-coordinate of the lower left corner of the frame
c    p1y0        y-coordinate of the lower left corner of the frame
c    allxl       =2 => all x labels/annotations are drawn (default)
c                =1 => only the x-label/annotations on the bottom row
c                      of frames on the page are drawn.
c                =0 => labels aren't even drawn on the bottom row.
c                if allxl < 0, same as abs(allxl) except last annotation
c                is not drawn if it falls at xxmax.
c    allyl       =2 => all y labels/annotations are drawn (default)
c                =1 => only the y-label/annotations on the left column
c                      of frames on the page are drawn.
c                =0 => labels aren't even drawn on the left column.
c                if allyl < 0, same as abs(allyl) except last annotation
c                is not drawn if it falls at yymax.
c    np1h        number of frames to be arranged horizontally per page
c    ip1h        in the current row   , this is the ip1h'th frame
c    np1v        number of frames to be arranged vertically   per page
c    ip1v        in the current column, this is the ip1v'th frame
c    lnlg        = 1 => linear x and y;    = 2 => linear x, log y
c                = 3 => log x, linear y;   = 4 => log x and y
c
c  OUTPUT VARIABLES (via common block win1d):
c    scale       scaling factor for multiple frames per page
c    wxmin, etc. coordinates of plotting window in graphical units
c    xxmin, etc. coordinates of plotting window in "real" units after 
c                adjustments to "xmin", etc. have been made to allow x- 
c                and y-axes to start and end on major/minor tick marks
c
c  LOCAL VARIABLES:
c    cxann       1-D character array containing all x-axis annotations
c                in NCAR markup language.
c    cyann       1-D character array containing all y-axis annotations
c                in NCAR markup language.
c
c  EXTERNALS:
c    AXISSET1, AXISSET2
c    STRLEN
c    PSCHARS
c    AXISLAB
c    SLDLIN     )---- PSPLOT routine
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       integer       mann
       real          small
c       parameter     ( mann=20, small=0.001 )
       parameter     ( mann=20, small=0.05 )
c
       character*(*) xlab    , ylab
       integer       yexp    , imm     , allxl   , allyl   , np1h    
     1             , ip1h    , np1v    , ip1v    , lnlg    
     2             , aallxl  , aallyl  , ixsen   , iysen   
     3             , ndec    , iend    , i       , nch     , iexp    
     3             , nchmax  , ndiv    , nann    , i1      , ibeg
       real          xmin    , xmax    , ymin    , ymax    , aspect 
     1             , p1x0    , p1y0    , phw     , wx      , dwx     
     2             , dwx1    , dwx2    , wy      , dwy     , dwy1    
     3             , dwy2    , fact    , q1      , q2      , q3      
     4             , xx      , dxx     , yy      , dyy
       real          wxmin   , wxmax   , wymin   , wymax   , xxmin
     1             , xxmax   , yymin   , yymax   , scale
c
       character*32  cxann   (mann), cyann   (mann)
       real          xann    (mann), yann    (mann)
c
       common      / win1d /
     1               wxmin   , wxmax   , wymin   , wymax   , xxmin
     2             , xxmax   , yymin   , yymax   , scale
c
c      External statements
c
       integer       strlen
       external      axisset1, axisset2, strlen  , pschars , sldlin  
     1             , axislab
c
c-----------------------------------------------------------------------
c
c      Set axes sentinels.  If i*sen=1, *=x,y, *-label and annotations
c  are drawn.
c
       aallxl = abs ( allxl )
       ixsen  = 0
       if   (aallxl .eq. 2)                           ixsen = 1
       if ( (aallxl .eq. 1) .and. (ip1v  .eq. np1v) ) ixsen = 1
       aallyl = abs ( allyl )
       iysen  = 0
       if   (aallyl .eq. 2)                           iysen = 1
       if ( (aallyl .eq. 1) .and. (ip1h  .eq. 1   ) ) iysen = 1
c
c-----------------------------------------------------------------------
c----------------  W I N D O W   C O O R D I N A T E S  ----------------
c-----------------------------------------------------------------------
c
c      Compute window coordinates ("wxmin", etc.) and the scaling factor 
c  ("scale"), all in common block "win1d" declared also by GRFX1D.  
c
       scale = 1.0
       wxmin = p1x0
       wxmax = p1x0 + 10.0
       wymin = p1y0
       wymax = p1y0 + 10.0
       phw   = 5.0
c
       if (aspect .gt. 1.0) then
         wxmin = wxmin + phw * ( 1.0 - 1.0 / aspect )
         wxmax = wxmax - phw * ( 1.0 - 1.0 / aspect )
       endif
c
       if (aspect .lt. 1.0) then
         wymin = wymin + phw * ( 1.0 - aspect )
         wymax = wymax - phw * ( 1.0 - aspect )
       endif
c
c      Adjust for multiple frames per page.  The factor of 17.0 in 
c  "scale" allows room for the footer to be put at the bottom of the 
c  page in GRFX1D.
c
       dwx = 0.0
       dwy = 0.0
       if (np1h*np1v .gt. 1) then
         dwx   = 3.5
         dwy   = 2.2
         dwx1  = ( wxmax - wxmin + dwx ) * real ( np1h )
         dwy1  = ( wymax - wymin + dwx ) * real ( np1v )
         scale = 17.0 / max ( dwx1, dwy1 )
         if (aallyl .le. 1) then
           dwx  = 0.3
           dwx1 = ( wxmax - wxmin + dwx ) * real ( np1h ) + 3.2
         endif
         if (aallxl .le. 1) then
           dwy  = 0.3
           dwy1 = ( wymax - wymin + dwx ) * real ( np1v ) + 1.9
         endif
         wx    = 12.8
         wy    = 12.5
         dwx1  = ( wxmax - wxmin + dwx ) * scale
         dwx2  = ( wxmax - wxmin       ) * scale
         wxmax = wx - dwx1 * real ( np1h - ip1h ) + p1x0
         dwy1  = ( wymax - wymin + dwy ) * scale
         dwy2  = ( wymax - wymin       ) * scale
         wymax = wy - dwy1 * real ( ip1v - 1    ) + p1y0
         wxmin = wxmax - dwx2
         wymin = wymax - dwy2
c
       endif
c
c-----------------------------------------------------------------------
c------------------------  T H E   X - A X I S  ------------------------
c-----------------------------------------------------------------------
c
c      Set parameters for the x-axis.
c
       ndec =-1
       if (lnlg .le. 2) then
         call axisset1 ( 1, xmin, xmax, imm, aspect, ndec, iexp, nchmax
     1                 , xxmin, xxmax, ndiv, nann, xann, cxann )
       else
         call axisset2 ( 1, xmin, xmax, 0, aspect, nchmax
     1                 , xxmin, xxmax, ndiv, nann, xann, cxann )
         iexp = 0
       endif
c
c      Write the x-axis annotations and draw the annotated ticks.
c
       fact = ( wxmax - wxmin ) / ( xxmax - xxmin )
       wy   = wymin - 0.75  * scale
       q1   = 0.06 * scale
       q2   = 0.45 * scale
       q3   = 0.38 * scale
c
       iend = nann
       if (allxl .lt. 0) then
         q1 =  abs ( xxmax - xann(nann) ) / ( xxmax - xxmin )
         if (q1 .lt. small) iend = nann - 1
       endif
       do 20 i=1,iend
         wx = wxmin + ( xann(i) - xxmin ) * fact - q1
         if (ixsen .eq. 1) then
           nch = strlen ( cxann(i) )
           call pschars ( wx, wy, q2, cxann(i)(1:nch), 0.0, 1 )
         endif
         wx  = wx + q1
         call sldlin ( wx, wymin, wx, wymin+q3, 0.0 )
         call sldlin ( wx, wymax, wx, wymax-q3, 0.0 )
20     continue
c
c      Draw the unannotated ticks.
c
       q1 = 0.25  * scale
       if (ndiv .gt. 0) then
         dxx = ( xxmax - xxmin ) / real ( ndiv )
         do 30 i=2,ndiv
           xx = real ( i - 1 ) * dxx
           wx = wxmin + xx * fact
           call sldlin ( wx, wymin, wx, wymin+q1, 0. )
           call sldlin ( wx, wymax, wx, wymax-q1, 0. )
30       continue
       else
         xx = xxmin
40       continue
         i1 = int ( xx - 0.499 + sign ( 0.5, xx ) )
         xx = alog10 ( 10.0**xx + 10.0**i1 )
         if (xx .lt. xxmax) then
           wx = wxmin + ( xx - xxmin ) * fact
           call sldlin ( wx, wymin, wx, wymin+q1, 0. )
           call sldlin ( wx, wymax, wx, wymax-q1, 0. )
           go to 40
         endif
       endif
c
c      Draw and label the axis.
c
       call sldlin ( wxmin, wymin, wxmax, wymin, 0. )
       call sldlin ( wxmin, wymax, wxmax, wymax, 0. )
       if (ixsen .eq. 1)
     1   call axislab ( 1, wxmin, wxmax, wymin, wymax, scale, 1.0
     2                , nchmax, xlab, 1.0, iexp )
c
c-----------------------------------------------------------------------
c------------------------  T H E   Y - A X I S  ------------------------
c-----------------------------------------------------------------------
c
c      Set the parameters for the y-axis.
c
       ndec =-1
       if ( (lnlg .eq. 1) .or. (lnlg .eq. 3) ) then
         call axisset1 ( 2, ymin, ymax, imm, aspect, ndec, iexp, nchmax
     1                 , yymin, yymax, ndiv, nann, yann, cyann )
         iexp = iexp + yexp
       else
         call axisset2 ( 2, ymin, ymax, yexp, aspect, nchmax
     1                 , yymin, yymax, ndiv, nann, yann, cyann )
         iexp = 0
       endif
c
c      Write the y-axis annotations and draw the annotated ticks.  The
c  first y-axis annotation is skipped if both it and the first x-axis
c  annotation are at the origin.
c
       wx   = wxmin - 0.20  * scale
       fact = ( wymax - wymin ) / ( yymax - yymin )
       q1   = abs ( xann(1) - xxmin ) / ( xxmax - xxmin )
       q2   = abs ( yann(1) - yymin ) / ( yymax - yymin )
       ibeg = 1
       if ( (q1 .le. small) .and. (q2 .le. small) ) ibeg = 2
       q2   = 0.45 * scale
       q3   = 0.38 * scale
c
       iend = nann
       if (allyl .lt. 0) then
         q1 =  abs ( yymax - yann(nann) ) / ( yymax - yymin )
         if (q1 .lt. small) iend = nann - 1
       endif
       do 90 i=ibeg,iend
         wy = wymin + ( yann(i) - yymin ) * fact
         if (iysen .eq. 1) then
           nch = strlen ( cyann(i) )
           call pschars ( wx, wy-q2/2.0, q2, cyann(i)(1:nch), 0.0, 2 )
         endif
         call sldlin ( wxmin, wy, wxmin+q3, wy, 0.0 )
         call sldlin ( wxmax, wy, wxmax-q3, wy, 0.0 )
90     continue
c
c      Draw the unannotated ticks.
c
       q1 = 0.25  * scale
       if (ndiv .gt. 0) then
         dyy = ( yymax - yymin ) / real ( ndiv )
         do 100 i=2,ndiv
           yy = real ( i - 1 ) * dyy
           wy = wymin + yy * fact
           call sldlin ( wxmin, wy, wxmin+q1, wy, 0. )
           call sldlin ( wxmax, wy, wxmax-q1, wy, 0. )
100      continue
       else
         yy = yymin
110      continue
         i1 = int ( yy - 0.499 + sign ( 0.5, yy ) )
         yy = alog10 ( 10.0**yy + 10.0**i1 )
         if (yy .lt. yymax) then
           wy = wymin + ( yy - yymin ) * fact
           call sldlin ( wxmin, wy, wxmin+q1, wy, 0. )
           call sldlin ( wxmax, wy, wxmax-q1, wy, 0. )
           go to 110
         endif
       endif
c
c      Draw and label the axis.
c
       call sldlin ( wxmin, wymin, wxmin, wymax, 0. )
       call sldlin ( wxmax, wymin, wxmax, wymax, 0. )
       if (iysen .eq. 1)
     1   call axislab ( 2, wxmin, wxmax, wymin, wymax, scale, 1.0
     2                , nchmax, ylab, 1.0, iexp )
c
c-----------------------------------------------------------------------
c
c      Final adjustments for log axes.
c
       if (lnlg .ge. 3) then
         xxmin = 10.0**xxmin
         xxmax = 10.0**xxmax
       endif
       if ( (lnlg .eq. 2) .or. (lnlg .eq. 4) ) then
         yymin = 10.0**yymin
         yymax = 10.0**yymax
       endif
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine pschars ( x, y, fsize, chars, ang, just )
c
c    abm:grfx03.pschars <------------- emulate NCAR typesetting language
c                                                          october, 2003
c
c    written by: Alexander Men'shchikov
c    modified 1: February, 2005 by David Clarke, generalised for format 
c                statements 2050 and 2060 in GRFXCNTR.
c    modified 2: July, 2010 by DAC; completely rewritten to make it more 
c                readable and robust, and to enable sub-sub, sub-super, 
c                super-sub, and super-super scripting.
c
c  PURPOSE: This subroutine emulates the NCAR library's typesetting
c  language used to draw labels and legends in plots, and is for use 
c  with the PSPLOT graphics library.
c
c  NCAR markup language:
c
c    Fonts: |F0|  = times-roman (default)
c           |F10| = times-roman italic
c           |F14| = times-roman italic bold
c           |F5|  = greek italic
c           |F18| = math characters
c           |F34| = more math characters
c
c    Position: |B| => subscript; |S| => superscript; |N| => normal
c
c    One may combine one position designator with one font change in
c    each "field" (bounded by ||), and then string as many fields
c    together as one needs
c
c  Note: this routine is not general, and works based on several 
c  formatting assumptions.  
c
c  1.  Position control characters (B, S, N) may appear in the same 
c      control sequence as the font specifier (e.g., F0) but must appear 
c      before the font specifier.  Thus, |BF0| is OK, |F0B| is not.  
c
c  2.  There may be one S, one B, or one or two Ns before a font 
c      specifier.  Thus, |SF0| and |NNF0| are OK, |NSF0| is not.
c
c  3.  Some key characters are sensitive to extraneous blanks.  Thus, 
c      and for example, |F18|U gives a multiplication symbol, |F18| U 
c      gives a 'Y'.
c
c  INPUT VARIABLES:
c    x,y        position coordinates (in grid units) of the first 
c               character sequence printed.  (999.,999.) => current pen 
c               position.
c    fsize      font size (in cm?)
c    chars      character string (in NCAR markup language) to be parsed 
c               and interpreted
c    ang        angle of text.  Currently, allowed angles are 0 (x-axis) 
c               and 90 (y-axis).  With a little work, any angle could be 
c               interpreted as PSPLOT doesn't restrict angles to 0 & 90.
c    just       =0 => (x,y) at left side of "text"
c               =1 => (x,y) at centre of "text"
c               =2 => (x,y) at right side of "text"
c               where "text" is the entire character string for (x,y)
c              .ne. (999.,999.), only the first character "token" 
c               otherwise.
c
c  OUTPUT VARIABLES:
c
c  LOCAL VARIABLES:
c
c  EXTERNALS:
c    STRLEN
c    KEKSYM   )
c    SUPER    )---- PSPLOT routines
c    SUBBER   )
c    SETFNT   )
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       integer       nlim
       parameter     ( nlim=100 )
c
       character*(*) chars
       character*128 word
       integer       i       , j       , k       , n       , nch
     1             , nch2    , i1      , i2      , nfont   , nfnt    
     2             , just    , k0      , kmax    , nmax    , lev
       real          x       , y       , xi      , yi      , fsize
     1             , fs      , ang     , width   , fact
c
       character*128 control (nlim), token   (nlim)
       integer       tlen    (nlim), idc     (nlim), level   (nlim)
c
c      External statements
c
       integer       strlen
       external      strlen  , keksym  , super   , subber  , setfnt
c
c-----------------------------------------------------------------------
c
       do 10 i=1,nlim
         control(i) = ''
         token  (i) = ''
         tlen   (i) = 0
10     continue
c
c      Find the location of all delineation characters (|) in the 
c  string.
c
       nch = strlen ( chars )
       n   = 0
       do 20 i=1,nch
         if (chars(i:i) .eq. '|') then
           n      = n + 1
           idc(n) = i
         endif
20     continue
       nmax = n
       if (nmax .eq. 0) then
c
c      If no delineation characters were found, assume the entire string 
c  is to be printed as is at the default font (F0 => nfnt=24).
c
         control(1) = 'F0'
         tlen   (1) = nch
         token  (1) = chars(1:nch)
         kmax       = 1
c
       else
c
c      If the first character (including blanks!) is not a |, assume 
c  this string is the first token to be printed at the default font, F0.
c
         k0 = 0
         if (idc(1) .gt. 1) then
           control(1) = 'F0'
           tlen   (1) = idc(1)-1
           token  (1) = chars(1:tlen(1))
           k0         = 1
         endif
c
c      For syntactically correct NCAR markup language, control codes 
c  follow an odd number of delineation characters, while character 
c  substrings to be printed (tokens) follow an even number of 
c  delineation characters.
c
         do 30 n=1,nmax
           k  = n/2
           i1 = idc(n) + 1
           i2 = nch
           if (n .lt. nmax) i2 = idc(n+1) - 1
           if (i2 .ge. i1) then
             if (n .eq. 2*k) then
               token  (k+k0  ) = chars(i1:i2)
               tlen   (k+k0  ) = i2 - i1 + 1
             else
               control(k+k0+1) = chars(i1:i2)
             endif
           endif
30       continue
         kmax = nmax/2 + k0
c
c      Squeeze out any blanks that may be in the control sequences.
c
         do 50 k=1,kmax
           write ( word, '(64x)' )
           i   = 0
           nch2 = strlen ( control(k) )
           do 40 n=1,nch2
             if (control(k)(n:n) .ne. ' ') then
               i = i + 1
               word(i:i) = control(k)(n:n)
             endif
40         continue
           control(k) = word
           if ( (word(1:1) .ne. 'F') .and. (word(1:1) .ne. 'N') .and. 
     1          (word(1:1) .ne. 'S') .and. (word(1:1) .ne. 'B') ) then
             write (6, 2010) word(1:i), chars(1:nch)
             control(k) = 'NF0'
           endif
50       continue
c
       endif
c
c      Determine the text level (sub-superscripting) applicable for each 
c  token.  Text can be at any one of seven positions:
c
c  lev = -3 => sub-subscript                     1 => super-subscript
c        -2 => subscript         0 => normal     2 => superscript
c        -1 => sub-superscrpt                    3 => super-superscript
c
       lev = 0
       do 60 k=1,kmax
         if (control(k)(1:1) .eq. 'N') then
           if (abs(lev) .eq. 2) lev = 0
           if ( (lev .eq. 1) .or. (lev .eq. 3) ) lev = 2
           if ( (lev .eq.-1) .or. (lev .eq.-3) ) lev =-2
         endif
         if (control(k)(2:2) .eq. 'N') then
           if (abs(lev) .eq. 2) lev = 0
           if ( (lev .eq. 1) .or. (lev .eq. 3) ) lev = 2
           if ( (lev .eq.-1) .or. (lev .eq.-3) ) lev =-2
         endif
         if (control(k)(1:1) .eq. 'B') then
           if (abs(lev) .eq. 2) lev = lev - 1
           if (lev .eq. 0) lev =-2
         endif
         if (control(k)(1:1) .eq. 'S') then
           if (abs(lev) .eq. 2) lev = lev + 1
           if (lev .eq. 0) lev = 2
         endif
         level(k) = lev
60     continue
c
c      The first token is centred according to "just" after which the 
c  remaining tokens are appended.  This is fine for left-justification
c  but not for centring or right-justification.  For the latter two, it 
c  is necessary to adjust the initial coordinates (xi,yi), which is done 
c  by estimating the length of text to be printed, including an account 
c  of the smaller characters that may be sub-superscripted.
c
       xi = x
       yi = y
       if (kmax .gt. 1) then
         width = 0.0
         do 70 k=2,kmax
           fact = 0.4
           if (    level(k)  .eq. 0) fact = 0.9
           if (abs(level(k)) .eq. 2) fact = 0.6
           width = width + fact * fsize * real ( tlen(k) )
           do 61 i=1,tlen(k)
             if (token(k)(i:i) .eq. '.') 
     1         width = width - 0.5 * fact * fsize
61         continue
70       continue
         if ( (ang .eq. 0.0) .and. (x .ne. 999.0) ) then
           if (just .eq. 1) xi = x - width * 0.5
           if (just .eq. 2) xi = x - width
         endif
         if ( (ang .eq. 90.0) .and. (y .ne. 999.0) ) then
           if (just .eq. 1) yi = y - width * 0.5
           if (just .eq. 2) yi = y - width
         endif
       endif
c
c      Print each token with the specified font and at the specified 
c  sub-superscript level.
c
       do 80 k=1,kmax
c
c      Set font.
c
         if ( (control(k)(1:1) .eq. 'F') .or.
     1        (control(k)(2:2) .eq. 'F').or.
     1        (control(k)(3:3) .eq. 'F') ) then
c
           if (control(k)(1:1) .eq. 'F') then
             read (control(k)(2:),'(i2)') nfont
           else
             if (control(k)(2:2) .eq. 'F') then
               read (control(k)(3:),'(i2)') nfont
             else
               read (control(k)(4:),'(i2)') nfont
             endif
           endif
c
c      Substitution of characters that are different in NCAR and PSPLOT
c
           if ( (nfont .eq. 18) .and. (token(k)(1:1) .eq. 'R') ) then
             token(k) = '||'
             tlen (k) = 2
           endif
           if ( (nfont .eq. 18) .and. (token(k)(1:1) .eq. 'U') ) then
             token(k) = '264'
             tlen (k) = -999
           endif
           if ( (nfont .eq. 18) .and. (token(k)(1:1) .eq. 'A') ) then
             token(k) = '136'
             tlen (k) = -999
           endif
           if ( (nfont .eq. 34) .and. (token(k)(1:1) .eq. 'Q') ) then
             token(k) = '321'
             tlen (k) = -999
           endif
           if ( (nfont .eq. 34) .and. (token(k)(1:1) .eq. 'W') ) then
             token(k) = '327'
             tlen (k) = -999
           endif
           if ( (nfont .eq. 34) .and. (token(k)(1:1) .eq. '0') ) then
             token(k) = '260'
             tlen (k) = -999
           endif
c
           nfnt =-1
c
c      default is NewCenturySchlbk-Roman
c
           if (nfont .eq.  0) nfnt = 24
c
c      Italic --> NewCenturySchlbk-Italic
c
           if (nfont .eq. 10) nfnt = 23
c
c      Bold --> NewCenturySchlbk-Bold
c
           if (nfont .eq. 13) nfnt = 21
c
c      Bold italic --> NewCenturySchlbk-BoldItalic
c
           if (nfont .eq. 14) nfnt = 22
c
c      Greek --> Symbol
c
           if (nfont .eq.  5) nfnt = 29
c
c      Symbol Set 1 --> Symbol
c
           if (nfont .eq. 18) nfnt = 29
c
c      Math Symbols --> Symbol
c
           if (nfont .eq. 34) nfnt = 29
c
           if (nfnt .eq. -1) then
             write (6, 2020) nfont, chars(1:nch)
             nfnt = 24
           endif
           call setfnt ( nfnt )
c
         endif
c
c      Font size.
c
         fs = fsize
         if (abs(level(k)) .eq. 1) fs = 2.0 * fsize / 3.0
         if (abs(level(k)) .eq. 3) fs = 2.0 * fsize / 3.0
c
c      Additional positional shift for secondary sub-superscripting.
c
         fact = 1.0
         if (level(k) .eq.-3) fact = 2.5
         if (level(k) .eq.-1) fact = 0.08
         if (level(k) .eq. 1) fact = 0.8
         if (level(k) .eq. 3) fact = 2.5
c
c      Write text.
c
         if (level(k) .ge. 1) then
           call super  ( token(k), tlen(k), fs, ang, fact )
         endif
         if (level(k) .eq. 0) then
           call keksym ( xi, yi, fs, token(k), ang, tlen(k), just )
         endif
         if (level(k) .le.-1) then
           call subber ( token(k), tlen(k), fs, ang, fact )
         endif
c
         xi = 999.0
         yi = 999.0
80     continue
c
c      Restore default font.
c
       call setfnt ( 24 )
c
c-----------------------------------------------------------------------
c----------------------- Write format statements -----------------------
c-----------------------------------------------------------------------
c
2010   format('PSCHARS : |---WARNING---> Unrecognised control '
     1       ,'sequence "',a,'" in input string:',/
     2       ,'PSCHARS : "',a,'"',/
     3       ,'PSCHARS : Recognised control sequences begin with "F", '
     4       ,'"N", "S", or "B".',/
     5       ,'PSCHARS : Resetting control sequence to "NF0"; '
     6       ,'execution continuing...')
2020   format('PSCHARS : |---WARNING---> Unknown font F',i2,' in input '
     1       ,'character string:',/
     2       ,'PSCHARS : "',a,'"',/
     3       ,'PSCHARS : Recognised fonts include F0, F5, F10, F13, '
     4       ,'F14, F18, and F34.',/
     5       ,'PSCHARS : Resetting to default font, F0; execution '
     6       ,'continuing...')
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       function strlen ( word )
c
c    dac:grfx03.strlen <--------------------- length of character string
c    from dac:editor.strlen                               february, 1992
c
c    written by: David Clarke
c    modified 1:
c
c  PURPOSE: This function returns the number of characters in the
c  character string "word".  Note that PSPLOT has a function LENSTR with 
c  essentially the same purpose.
c
c  INPUT VARIABLES:
c    word        input character*(*) string
c
c  OUTPUT VARIABLES:
c    strlen      number of characters up to and including the last
c                non-blank character in "word"
c
c  LOCAL VARIABLES:
c
c  EXTERNALS: [NONE]
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) word
       integer       strlen  , i       , imax
c
c-----------------------------------------------------------------------
c
       imax   = len ( word )
       strlen = 0
       do 10 i=1,imax
         if (word(i:i) .ne. ' ') strlen = i
10     continue
c
       return
       end
c
c=======================================================================
c=======================================================================
c
       subroutine timedate ( chtime, chdate )
c
c    dac:plotit.timedate <------------------------ returns time and date 
c    from dac:zeus3d.wclock                                   june, 2018
c
c    written by: David Clarke
c    modified 1:
c
c  PURPOSE: This routine returns the wall-clock time (hh:mm:ss) and date 
c  (dd/mm/yyyy) in character variables; a pared-down version of WCLOCK 
c  in dzeus36 meant only for gfortran, ifort, and pgfortran (not pgf77 
c  or pgf95).
c
c  INPUT VARIABLES:
c
c  OUTPUT VARIABLES:
c    chtime      time returned as hh:mm:ss
c    chdate      date returned as dd/mm/yyyy
c
c  LOCAL VARIABLES:
c
c  EXTERNALS: [none]
c
c-----------------------------------------------------------------------
c
       implicit      none
c
       character*(*) chtime  , chdate
       character*10  chstr
       integer       year    , month   , day     , hour    , minute  
     1             , second  , ncht    , nchd    , nch     
c
       integer       values  (   8)
c
c-----------------------------------------------------------------------
c
c  DATE_AND_TIME returns eight integers in "values", where:
c
c  values(1)   the year (e.g., 2011)
c  values(2)   the month of the year (1-12)
c  values(3)   the day of the month (1-31)
c  values(4)   the number of minutes from GMT (e.g., -240 for Halifax)
c  values(5)   the hour of the day (0-23)
c  values(6)   the minute of the hour (0-59)
c  values(7)   the second of the minute (0-59)
c  values(8)   the number of milliseconds of the second (0-999)
c
       call date_and_time ( values=values )
       year   = values(1)
       month  = values(2)
       day    = values(3)
       hour   = values(5)
       minute = values(6)
       second = values(7)
c
c      Set "chtime" and "chdate"
c
       ncht   = len ( chtime )
       chtime = ' '
       write ( chstr, 2010 ) hour, minute, second
       nch           = min ( 8, ncht )
       chtime(1:nch) = chstr(1:nch)
c
       nchd   = len ( chdate )
       chdate = ' '
       write ( chstr, 2020 ) day, month, year
       nch           = min ( 10, nchd )
       chdate(1:nch) = chstr(1:nch)
c
c-----------------------------------------------------------------------
c
2010   format(i2.2,':',i2.2,':',i2.2)
2020   format(i2,'/',i2.2,'/',i4.4)
c
       return
       end
c
c=======================================================================
c
c+++++++++++++++++++++++++++ PSPLOT ROUTINES +++++++++++++++++++++++++++
c
c  The following modules represent a subset of the full PSPLOT library 
c  (/Users/dclarke/grfx/psplot) that support simple line graphs.  No 2-D 
c  colour or gray-scale contouring routines are included.
c
c  All PSPLOT modules were kindly provided by the late Kevin Kohler,
c  formerly of Nova Southeastern University, Fort Lauderdale, FL.
c
c  Modifications to individual routines made by Alexander Men'shchikov 
c  and David Clarke.
c
c=======================================================================
       subroutine asterisk ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by DAC, June 2010 using dk times as a template:  This routine 
c  draws an asterisk symbol on a 1 - D plot.
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius
c
       real          xa      (   2), ya      (   2)
c
       external      drwcrv  
c----------------------------------------------------------------------- 
       xa(1) = xc
       ya(1) = yc - radius
       xa(2) = xc
       ya(2) = yc + radius
       call drwcrv ( xa, ya, 2, 0.0, .false. )
c
       xa(1) = xc - 0.87 * radius
       ya(1) = yc - 0.50 * radius
       xa(2) = xc + 0.87 * radius
       ya(2) = yc + 0.50 * radius
       call drwcrv ( xa, ya, 2, 0.0, .false. )
c
       ya(1) = yc + 0.50 * radius
       ya(2) = yc - 0.50 * radius
       call drwcrv ( xa, ya, 2, 0.0, .false. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine blkstp ( ch, ndim, a, leng )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine strips out blanks
c
c  idim        leading dimension of arr in calling program
c  ch          input character*1 array
c  ndim        dimension of "ch" and "a"
c  leng        number of non-blank characters
c----------------------------------------------------------------------- 
       implicit      none
c
       integer       ndim    , leng    , i       , l
c
       character*1   ch      (ndim), a       (ndim)
c----------------------------------------------------------------------- 
c
c      Strip out blanks only (leave in esc, etc.)
c
       i    = 1
       leng = 0
10     continue
       if ( ichar(ch(i)) .ne. 32 )then
         leng    = leng + 1
         a(leng) = ch(i)
       endif
c
       if ( i .eq. ndim ) then
c
c      Blankfill remainder of output array
c
         do 20 l=leng+1,ndim
           a(l) = ' '
20       continue
         go to 30
       endif
c
       i = i + 1
       go to 10
30     continue
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine circle ( xc, yc, radius, fill )
c----------------------------------------------------------------------- 
c  written by KEK: writes a (filled) circle as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of circle
c  radius      radius of circle (inches)
c  fill        logical toggle indicating if circle is to be filled
c----------------------------------------------------------------------- 
       implicit      none
c
       logical        fill
       character*132  cmdstr  , scrc
       real           xc      , yc      , radius  , conver  , xci     
     1              , yci     , rad
c
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
c
       integer       lenstr
       external      lenstr  , filler
c----------------------------------------------------------------------- 
       xci  = xc     * conver
       yci  = yc     * conver
       rad  = radius * conver
       scrc = ' '
       write ( scrc, '(f8.2,'' '',f8.2,'' '',f8.2,'' C'')' )
     1   xci, yci, rad
       if ( fill ) then
         cmdstr = 'Np '//scrc(1:lenstr(scrc,132))//' fill'
       else
         cmdstr = 'Np '//scrc(1:lenstr(scrc,132))//' stroke'
       endif
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
c----------------------------------------------------------------------- 
       subroutine dshcrv ( x, y, npts, idshpn, thk )
c----------------------------------------------------------------------- 
c  Written by KEK: this routine draws a dashed curve connecting points 
c  (x(n),y(n)), n=1,npts.
c
c  x,y         coordinate arrays in inches
c  npts        dimension of x and y; number of points to connect
c              starting at (x(1),y(1))
c  idshpn      number of solid/blank patterns per inch.  0 => solid 
c              curve is drawn.
c  thk         thickness of the dashes (inches).  0.0 => a single line
c              is drawn for the dashes
c----------------------------------------------------------------------- 
       implicit      none
c
       integer       npts    , idshpn  , iset    , ipen    , n       
     1             , nn      
       real          thk     , curwid  , curwids , pi      , absx    
     1             , ang     , dist    , xe      , xnew    , ye      
     2             , ynew
       real*8        dlen    , dleft   , xsd     , ysd     , xed
     1             , yed
c
       real          x       (npts), y       (npts)
 
       common      / lcom     / curwid
c
       external      drwcrv  , plot
c
       integer       icall
       data          icall    / 1 /
       save          icall
c----------------------------------------------------------------------- 
       if (icall .eq. 0) then
         write(6,3000)
3000     format('PSPLOT: EXTERNAL: DSHCRV')
         icall = 1
       endif
c
       if (idshpn .le. 0) then
         call drwcrv ( x, y, npts, thk, .false. )
         return
       endif
c
       pi = acos (-1.0)
c
       iset = 0
       if ( (thk .ne. 0.0) .and. (thk.ne.curwid) ) then 
         iset    = 1
         curwids = curwid
         call setlw ( thk )
       endif
c
       dlen = 0.5d0 / dble(idshpn)
       ipen = 3
       call plot ( x(1), y(1), ipen )
       xsd  = dble(x(1))
       ysd  = dble(y(1))
       n    = 2
c
10     continue
       if ( n .gt. npts ) go to 30         ! we're done
       ipen = 5 - ipen                     !(3-->2, 2-->3)
c
c      Find angle to next data point.
c
       absx = abs ( x(n) - real(xsd) )
       if ( absx .lt. 1.0e-5 ) then
         ang = pi/2.0 * sign ( 1.0, y(n) - real(ysd) )
       else
         ang = atan2 ( y(n) - real(ysd), x(n) - real(xsd) )
       endif
c
c      Calculate end point of dash or space.
c
       xed = xsd + dlen * dble ( cos(ang) )
       yed = ysd + dlen * dble ( sin(ang) )
       xe  = real ( xed )
       ye  = real ( yed )
c
c      See if we can fit this in before next data point.
c
       dist = sqrt ( (x(n) - real(xsd) )**2 + ( y(n) - real(ysd) )**2 )
       if ( dist .le. dlen ) then
c
c      Can't fit
c
         dleft = dlen
         do 20 nn=n,npts
c
c      Find angle to next data point.
c
           absx = abs ( x(nn) - real(xsd) )
           if ( absx .lt. 1.0e-5 ) then
             ang = pi/2.0 * sign ( 1.0, y(nn) - real(ysd) )
           else
             ang = atan2 ( ( y(nn) - real(ysd) )
     1                   , ( x(nn) - real(xsd) ) )
           endif
           dist = sqrt ( ( x(nn) - real(xsd) )**2 
     1                 + ( y(nn) - real(ysd) )**2 )
           if ( dist .le. dleft ) then
c
c      Draw line to next data point and calculate rest.
c
             call plot ( x(nn), y(nn), ipen )
             xsd   = dble ( x(nn) )
             ysd   = dble ( y(nn) )
             dleft = dleft - dble ( dist )     !Remaining portion
             if ( dleft .eq. 0.0d0 ) then 
c
c      Exact fit, so just do next point
c
               n = nn + 1
               go to 10
             endif
           else
c
c      Draw remaining portion and move toward.
c
             xnew = real ( xsd + dleft * dcos ( dble(ang) ) )
             ynew = real ( ysd + dleft * dsin ( dble(ang) ) )
             call plot ( xnew, ynew, ipen )
             xsd  = dble ( xnew )
             ysd  = dble ( ynew )
             n    = nn
             go to 10
           endif
20       continue
       else
c
c      Can fit, so just draw normally.  Check to see if we should just 
c  move to data point itself since roundoff could cause weird angles.
c
         dist = sqrt ( ( x(n) - real(xed) )**2
     1               + ( y(n) - real(yed) )**2 )
         if ( dist .le. 1.0e-5 ) then
           xed = dble(x(n))
           yed = dble(y(n))
           xe  = real(xed)
           ye  = real(yed)
           n   = n + 1
         endif
         call plot ( xe, ye, ipen )
         xsd = dble ( xed )
         ysd = dble ( yed )
         go to 10
       endif
c
30     continue
c
c      Reset linewidth to original
c
       if ( iset .eq. 1 ) call setlw ( curwids )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine diamond ( xc, yc, height )
c----------------------------------------------------------------------- 
c  written by DAC, June 2010 using dk times as a template:  This routine 
c  draws a diamond symbol on a 1-D plot.
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , height  , width
c
       real          xa      (   4), ya      (   4)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       width = 0.8 * height
       xa(1) = xc - width
       ya(1) = yc
       xa(2) = xc
       ya(2) = yc - height
       xa(3) = xc + width
       ya(3) = yc
       xa(4) = xc
       ya(4) = yc + height
       call drwcrv ( xa, ya, 4, 0.0, .true. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine drwcrv ( xarr, yarr, n, thkk, closer )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine draws a curved line on the current plot.
c
c  xarr, yarr  coordinates through which curve is drawn
c  n           dimension of xarr, yarr
c  thkk        desired thickness of line; 0.0 => use current thickness
c  closer      .true.  => dashed line
c              .false. => solid line
c----------------------------------------------------------------------- 
       implicit      none
c
       logical       closer
       character*132 cmdstr
       integer       n       , nn
       real          thkk    , curwids , curwid
c
       real          xarr    (   n), yarr    (   n)
c
c      "curwid" is set in routine setlw
c
       common      / lcom     / curwid
       common      / plt1     / cmdstr
c
       external      filler  , setlw   , movet   , linet   , stroke
c----------------------------------------------------------------------- 
       cmdstr = 'Np'
       call filler
c
       if ( thkk .ne. 0.0 ) then
         curwids = curwid
         call setlw ( thkk )
       endif
       call movet ( xarr(1), yarr(1) )
       do 10 nn=2,n
         call linet ( xarr(nn), yarr(nn) )
         if ( mod ( nn, 100 ) .eq. 0 ) then
c
c      stroke now to avoid possible limit
c
           call stroke
           call movet ( xarr(nn), yarr(nn) )
         endif
10     continue
c
       if ( closer ) then
         cmdstr = 'Cs'
       else
         cmdstr = 'S'
       endif
       call filler
       if ( thkk .ne. 0.0 ) call setlw ( curwids )   !Reset linewidth
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine factor ( facc )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine ... 
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          facc    , fac     , recipx  , recipy
c
       common      / plt1     / cmdstr
       common      / plt2     / fac
c
       external      filler
c----------------------------------------------------------------------- 
c
c      Unscale previous scaling
c
       recipx = 1.0 / fac
       recipy = 1.0 / fac
       write ( cmdstr, '(2f7.3,'' scale'')' ) recipx, recipy
       call filler
       fac = facc
       write ( cmdstr, '(2F7.3,'' scale'')' ) fac, fac
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine filler
c----------------------------------------------------------------------- 
c  Written by KEK: this routine...
c----------------------------------------------------------------------- 
       implicit      none
c
       logical       ispace
       character*132 cmdstr
       integer       iunit   , ibslash , lc      , lcc     , itot
     1             , icclst  , l       , icc     , ii
c
       character*1   cmdc    ( 132)
c
       common      / plt1     / cmdstr
       common      / pspunit  / iunit
c
       integer       lenstr
       external      lenstr
c----------------------------------------------------------------------- 
       ibslash = 92
       lc      = 0
       lcc     = lenstr ( cmdstr, 132 )
       ispace  = .false.
c
c      "itot" is running total of left/right parentheses in text string
c      If itot=0 then we are not in text mode, i.e. left = right
c
       itot   = 0
       icclst =-999
       do 10 l=1,lcc
         icc = ichar ( cmdstr(l:l) )
c
c      Don't place 2 or more spaces together not in text mode.
c
         if ( (icc .eq. 32) .and. (ispace) .and. (itot .eq. 0) )
     1     go to 10
         if ( (icc .ge. 32) .and. (icc .le. 127) ) then
           lc       = lc + 1
           cmdc(lc) = cmdstr(l:l)
         endif
         if (icc .eq. 32) then
           ispace = .true.
         else
           ispace = .false.
         endif
         if (icclst .ne. ibslash) then
           if (icc .eq. 40) itot = itot + 1
           if (icc .eq. 41) itot = itot - 1
         endif
         icclst = icc
10     continue
c
c      Write cmdstr to output file
c
       write ( iunit, '(132a1)' ) ( cmdc(ii), ii=1,lc )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine keksym ( xp, yp, size, titlein, ang, nchar1, mjus )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine ...
c
c  Modified by ABM, 10/03: converted Hollerith strings to character 
c  strings; generalized to accept special characters (fonts 29 and 35)
c----------------------------------------------------------------------- 
       implicit      none
c
       character*1   bslash
       character*132 cmdstr  , curfnt  , scrc    , titleb  , titlec
       character*(*) titlein
       integer       nchar1  , mjus    , nchar   , mchar   , iht
     1             , ifntsz  , nfont   , ititle  , m       , njus
     2             , numc    
       real          xp      , yp      , size    , ang     , conver  
     1             , pi      , high    , rsize   , szrat   , wide    
     2             , xarg    , yarg    , xpos    , ypos    , arg     
     3             , argdeg
c
       character*4  ctitle(20)
c
       common      / fntcom   / curfnt  , ifntsz  , nfont
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
       common      / kkplot   / szrat
c
       integer       lenstr
       external      filler  , blkstp  , lenstr
c----------------------------------------------------------------------- 
c
c      Stroke previous paths before this write
c
       cmdstr = 'S'
       call filler
       bslash = char(92)
       pi     = acos (-1.0)
c
       if ( nchar1 .eq. -999 ) then        ! octal code
         ctitle(1) = titlein
         read ( ctitle(1), '(I4)' ) ititle
         nchar     = 1
       else
         nchar     = nchar1
         titlec    = titlein
         if ( iabs(nchar) .lt. 132 ) titlec(nchar+1:132) = ' '
         read ( titlec, '(20a4)' ) ctitle
       endif
c
c      Choose proper font height, using current font.
c
       mchar = iabs ( nchar )
c
c      Set character size
c
       iht = int ( size * conver / 0.6 )     !0.6 factor is empirical
       if ( iht .ne. ifntsz ) then
         cmdstr = ' '
         write ( cmdstr, '(I3,'' Setf'')' ) iht
         call filler
         ifntsz = iht
       endif
c
       if ( nchar1 .eq. -999 ) then        !Octal code
         write ( titlec, '(A1,I10)' ) bslash, ititle
         call blkstp ( titlec, 132, titlec, numc )
       else
         write ( titlec, '(20a4)' ) ctitle
c
c      Check if "titlec" contains (, ), or "backslash." These characters 
c  must be treated specially by preceding them with a "backslash". Do 
c  this to ( and ) even though they might be balanced, i.e. () within a 
c  string, which can be treated normally.
c
         titleb = titlec
         numc   = 0
         do 10 m=1,mchar
           if ( (titleb(m:m) .eq. '(') .or. (titleb(m:m) .eq. ')') .or.
     1          (titleb(m:m) .eq. bslash) ) then
             numc              = numc + 1
             titlec(numc:numc) = bslash
           endif
           numc              = numc + 1
           titlec(numc:numc) = titleb(m:m)
10       continue
       endif
 
       mchar = numc
       xpos  = xp
       ypos  = yp
       if ( nchar .lt. 0 ) then
         njus = 0
       else
         njus = mjus
       endif
       rsize = size
c
       if ( xpos .eq. 999.0 ) then
         cmdstr = '/Xpos Xpres def'
         njus   = 0
       else
         cmdstr = ' '
         write ( cmdstr, '(f8.2,'' Xpd'')' ) xp * conver
       endif
       call filler
c
       if ( ypos .eq. 999.0 ) then
         cmdstr = '/Ypos Ypres def'
         njus   = 0
       else
         cmdstr = ' '
         write ( cmdstr, '(f8.2,'' Ypd'')' ) yp * conver
       endif
       call filler
c
       if ( (njus .ne. 0) .and. (njus .ne. 1) .and. (njus .ne. 2) ) then
         write ( *,2010 ) njus
2010     format (1x,'Incorrect justification code ',i5,'found in '
     1          ,'KEKSYMC, zero used')
         njus = 0
       endif
c
c      Strlen has already been "factored" by the choice of font height.
c  Since it will eventually be factored again, we must divide by factor 
c  now.
c
       cmdstr = '('//titlec(1:mchar)//') Le'
       arg  = ang * pi / 180.0
       xarg = 0.5 * cos(arg) * float (njus)
       yarg = 0.5 * sin(arg) * float (njus)
c
       if ( xarg .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(f7.3,'' Xpjd'')') xarg
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1            //scrc(1:lenstr(scrc,132))
       endif
c
       if ( yarg .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(f7.3,'' Ypjd'')' ) yarg
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1            //scrc(1:lenstr(scrc,132))
       endif
c
       if ( nchar .eq. -1 ) then    !centered symbol
         high = rsize
         wide = high * szrat
         xarg = 0.5 * ( high * sin(arg) - wide * cos(arg) )
         yarg =-0.5 * ( high * cos(arg) + wide * sin(arg) )
         xpos = xpos + xarg
         ypos = ypos + yarg
c
         if ( xarg .ne. 0.0 ) then
           scrc   = ' '
           write ( scrc, '(''/Xpos Xpos'',f8.2,'' add def'')' )
     1       xarg * conver
           cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1              //scrc(1:lenstr(scrc,132))
         endif
c
         if ( yarg .ne. 0.0 ) then
           scrc = ' '
           write ( scrc, '(''/Ypos Ypos'',f8.2,'' add def'')' )
     1       yarg * conver
           cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1              //scrc(1:lenstr(scrc,132))
         endif
       endif
c
       call filler
c
c      Move pen to proper coordinates
c
       scrc   = 'xyd mv'
       cmdstr = scrc(1:lenstr(scrc,132))
c
c      Set angle
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write(scrc,'(F7.1,'' rotate'')') ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1            //scrc(1:lenstr(scrc,132))
       endif
c
       scrc   = '('//titlec(1:mchar)//') show'
       cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1          //scrc(1:lenstr(scrc,132))
c
c      Reset angle
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(F7.1,'' rotate'')' ) -ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1            //scrc(1:lenstr(scrc,132))
       endif
       call filler
c
c      Start next char at 0.5 char width away
c
       argdeg = arg * 180.0 / pi
       cmdstr = ' '
       write ( cmdstr, '(f6.1,'' Xyps'')' ) argdeg
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
       function lenstr ( string, ls )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine finds actual length of string by 
c  eliminating trailing blanks
c
c  string      character string
c  ls          declared length of string in calling program
c----------------------------------------------------------------------- 
       implicit      none
c
       character*(*) string
       integer       lenstr  , ls      , i       , is
c----------------------------------------------------------------------- 
       do 10 i=ls,1,-1
         is = i
         if ( string(i:i) .ne. char(32) ) go to 20
10     continue
       is = 0
20     continue
       lenstr = is
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine linet ( xcall, ycall )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine is same as a call to plot with ip=2, but 
c  does not perform the "stroke".
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          xcall   , ycall   , conver
c
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
c
       external      filler
c----------------------------------------------------------------------- 
       write ( cmdstr, '(2f8.2,'' L'')' ) xcall * conver, ycall * conver
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine movet ( xcall, ycall )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine is same as call to plot with ip=3, but 
c  does not perform the "stroke".
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          xcall   , ycall   , conver
c
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
c
       external      filler
c----------------------------------------------------------------------- 
       write ( cmdstr, '(2f8.2,'' M'')' ) xcall * conver, ycall * conver
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine newdev ( iusnb )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine changes the output device/filename to 
c  the user specifications. It must be called before psinit.
c
c  iusnb       new file name ascii string
c----------------------------------------------------------------------- 
       implicit      none
c
       character*80  fileout
       character*(*) iusnb
       integer       inew    , lus
c
       common      / io       / fileout , inew
c
       integer       lenstr
       external      lenstr
c----------------------------------------------------------------------- 
       lus     = lenstr ( iusnb, len(iusnb) )
       fileout = ' '
       fileout = iusnb(1:lus)
c
c      Set inew to 999 so that init knows to use this name
c
       inew = 999
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine ocircle ( xc, yc, rad )
c----------------------------------------------------------------------- 
c  Written by DAC: This routine draws an open circle.
c
c  xc, yc      coordinates of circle centre
c  rad         radius of circle (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , rad
c
       external      circle
c----------------------------------------------------------------------- 
       call circle ( xc, yc, 0.95*rad, .false. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine plot ( xcall, ycall, ip )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine ...
c
c  idim        leading dimension of arr in calling program
c  xcall,ycall coordinates of...
c  ip          ...
c----------------------------------------------------------------------- 
       implicit      none
c
       character*80  scr
       character*132 cmdstr
       integer       ip      , iunit   , npage   , npageo  , ipp
     1             , nch
       real          xcall   , ycall   , conver
 
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
       common      / pspunit  / iunit
       common      / pagcom   / npage, npageo
c
       external      filler  , blkstp
c----------------------------------------------------------------------- 
       ipp = iabs(ip)
       if ( ip .eq. 999 ) then   !    Terminate plot session.
         cmdstr = 'stroke showpage'
         call filler
         cmdstr = '%%Trailer'
         call filler
         write ( scr, '(i6)' ) npage
         call blkstp ( scr, 80, scr, nch )
         cmdstr = '%%Pages: '//scr(1:nch)
         call filler
         cmdstr = '%%EOF'
         call filler
         npage  = npageo
         close(iunit)
         return
       endif
c
c      Moving pen
c
       if ( ipp .eq. 3 ) then 
c
c      Stroke to paint previous path, then move to...
c
         write ( cmdstr, '(2f8.2,'' SM'')' )
     1     xcall * conver, ycall * conver
       else
         write ( cmdstr, '(2f8.2,'' LM'')' )
     1     xcall * conver, ycall * conver
       endif
       call filler
c
c      Reset origin if ip < 0
c
       if (ip .lt. 0 ) then
         write ( cmdstr, '(2f8.2,'' translate'')' )
     1     xcall * conver, ycall * conver
         call filler
       endif
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine plus ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by DAC, 8/08 using dk square as a template: writes a plus 
c  sign as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of plus sign
c  radius      radius of plus sign (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius
c
       real          xa      (2), ya      (2)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       xa(1) = xc - radius
       ya(1) = yc
       xa(2) = xc + radius
       ya(2) = yc
       call drwcrv ( xa, ya, 2, 0.0, .false. )
       xa(1) = xc
       ya(1) = yc - radius
       xa(2) = xc
       ya(2) = yc + radius
       call drwcrv ( xa, ya, 2, 0.0, .false. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine psinit ( portrait )
c----------------------------------------------------------------------- 
c  Written by KEK: initializes plot for hp plotter
c
c  portrait    .true. => portrait orientation (.false. => landscape)
c----------------------------------------------------------------------- 
       implicit      none
c
       logical       portrait, prtrt   , iopen
       character*132 cmdstr  , curfnt
       character*80  fileout
       integer       ioffp   , inew    , ientry  , ifntsz  
     1             , nfont   , iunit   , npage   , npageo
       real          spval   , conver  , fac     , szrat   , xsh     
     1             , ysh
c
       common      / conre1   / ioffp   , spval
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
       common      / plt2     / fac
       common      / io       / fileout , inew
       common      / kkplot   / szrat
       common      / chpcom   / ientry  , prtrt
       common      / fntcom   / curfnt  , ifntsz  , nfont
       common      / pspunit  / iunit
       common      / pagcom   / npage   , npageo
c
       data          npageo   / 0 /
c
       integer       lenstr
       external      lenstr  , filler  , setfnt  , factor  , setlw
     1             , setgry  , setcolr , plot
c----------------------------------------------------------------------- 
       ioffp = 0
       spval = 0.0
c
c      Set conversion factor: conver=72.0 for inches; 72.0/2.54 for cm,
c  etc.
c
       conver = 72.0 / 2.54
       if (npageo .eq. 0) then
         npageo = 1
       else
         npageo = npage
       endif
       npage = 1
       prtrt = portrait
c
c      Use default name unless "newdev" has already been called (inew = 
c  999).
c
       if ( inew .eq. 0 ) then
         fileout = 'psplot.ps'
         inew    = 1
       endif
       if ( inew .eq. 999 )then
         inew    = 1
       endif
c
c      Open output file.
c
       iunit = 60
10     continue
       inquire ( iunit, opened=iopen )    ! get a non-existent unit
       if ( iopen ) then
         iunit = iunit + 1
         go to 10
       endif
c
       open ( iunit, file=fileout, status='unknown', form='formatted')
c
       cmdstr = '%!PS-Adobe-3.0'
       call filler
c
c      Next PS command introduced by Alexander Men'shchikov, Dec 2003.
c  It helps to avoid large white space at the top of a page when
c  displaying it with GhostView. This gives instant view of the entire 
c  plot, without any need to scroll it in the window. There may be a 
c  problem printing such a page, but one can switch Media in GhostView 
c  back to Letter or just delete/comment the line in the PS file.
c
c      cmdstr = '%%DocumentMedia: JustForScreen 585 652 80 white ( )'
c      call filler
c
       cmdstr =  '%%Title: '//fileout(1:lenstr(fileout,80))
       call filler
c
c       cmdstr =  '%%CreationDate: '//cdate//' '//ctime
c       call filler
c
       cmdstr =  '%%Creator: PSPLOT PostScript Plotting Package'
       call filler
c
       cmdstr =  '%%Pages: (atend)'
       call filler
c
       cmdstr = '%%EndComments'
       call filler
c
       cmdstr = '%Library Creator: Kevin E. Kohler <kevin@nova.edu>'
       call filler
c
       cmdstr = '%%BeginProlog'
       call filler
c
       cmdstr = '/inch {72 mul} bind def'
       call filler
c
       cmdstr = '/Ah {moveto lineto lineto stroke} def'
       call filler
       cmdstr = '/Ar {moveto 2 copy lineto 4  - 2 roll'
       call filler
       cmdstr = '     moveto lineto lineto stroke } def'
       call filler
c
       cmdstr = '/arcit {S /A2 exch def /A1 exch def /Rad exch def'
       call filler
       cmdstr = '        /Yc exch def /Xc exch def'
       call filler
       cmdstr = '        Xc Rad A1 cos mul add Yc Rad A1 sin mul add'
       call filler
       cmdstr = '        moveto newpath'
       call filler
       cmdstr = '        Xc Yc Rad A1 A2 arc stroke} def'
       call filler
c
       cmdstr = '/C {/Rad exch def /Yc exch def /Xc exch def'
       call filler
c
c      Closepath needed to avoid notch with fat line width
c
       cmdstr = '         Xc Yc Rad 0 360 arc closepath'
       call filler
       cmdstr = '        } def'
       call filler
c
       cmdstr = '/c0sf {closepath 0 setgray fill} def'
       call filler
c
       cmdstr = ' '
       write (cmdstr,'(''/cc {'',f7.4,'' mul def} def'')') conver
       call filler
c
       cmdstr = '/cd {cc /dy} def'
       call filler
c
       cmdstr = '/cf {closepath fill} def'
       call filler
c
       cmdstr = '/Cs {closepath stroke} def'
       call filler
c
       cmdstr = '/Cln {newpath 3 1 roll'
       call filler
       cmdstr = '      moveto {lineto} repeat clip newpath'
       call filler
       cmdstr = '     } def'
       call filler
c
       cmdstr = '/Cs {closepath stroke} def'
       call filler
c
       cmdstr = '/dd {/Dx dx def /Dy dy def} def'
       call filler
c
       cmdstr = '/Fb {newpath moveto '
       call filler
       cmdstr = ' Dx 0 rlineto 0 Dy rlineto Dx neg 0 rlineto closepath'
       call filler
       cmdstr = ' fill } def'
       call filler
c
       cmdstr = '/Fbn { newpath 3 1 roll moveto {lineto} repeat'
       call filler
       cmdstr = '       closepath fill } def'
       call filler
c
       cmdstr = '/Fbnc { newpath 3 1 roll moveto'
       call filler
       cmdstr = '       {lineto} repeat closepath fill } def'
       call filler
c
       cmdstr = '/L /lineto load def'
       call filler
c
       cmdstr = '/Le {/Strlen exch stringwidth pop def} def'
       call filler
c
c      Define stringlength slightly increased for integrand placement
c
       cmdstr = '/Lei {/Strlen exch stringwidth pop 1.5 mul def} def'
       call filler
c
c      Define stringlength slightly increased for summation placement
c
       cmdstr = '/Les {/Strlen exch stringwidth pop 1.1 mul def} def'
       call filler
c
       cmdstr = '/Lenssd '//
     1        '{/Strlenss exch stringwidth pop 3 mul 4 div def} def'
       call filler
c
       cmdstr = '/LM {2 copy lineto stroke moveto} def'
       call filler
c
       cmdstr = '/lm {Xp Yp lineto stroke mv} def'
       call filler
c
       cmdstr = '/M /moveto load def'
       call filler
c
       cmdstr = '/mv {Xp Yp moveto} def'
       call filler
c
       cmdstr = '/Np {newpath} def'
       call filler
c
       cmdstr = '/S /stroke load def'
       call filler
c
       cmdstr = '/Sc {setrgbcolor} def'
       call filler
c
       cmdstr = '/Sg {setgray} def'
       call filler
c
       cmdstr = '/Setf {Curfnt exch scalefont setfont} def'
       call filler
c
       cmdstr = '/SM {stroke moveto} def'
       call filler
c
       cmdstr = '/sm {stroke mv} def'
       call filler
c
       write ( cmdstr, '(''/Slw {'',f7.4,'' mul setlinewidth} def'')' )
     1   conver
       call filler
c
c      Minimum line width 300 dpi
c
       cmdstr = '/Slw0 {.24 setlinewidth} bind def'
       call filler
c
       cmdstr = '/Xpd {/Xpos exch def} def'
       call filler
c
       cmdstr = '/Xpjd '//
     1        ' {/Xpos exch Xpos exch Strlen mul sub def} def'
       call filler
c
       cmdstr = '/xyd {/Xp Xpos def /Yp Ypos def} def'
       call filler
c
       cmdstr = '/Xyp0d {/Xpos0 Xpres def /Ypos0 Ypres def} def'
       call filler
c
       cmdstr = '/Xyps {dup '//
     1        '/Xpres exch cos Strlen mul Xpos add def'
       call filler
       cmdstr = '              '//
     1        '/Ypres exch sin Strlen mul Ypos add def} def'
       call filler
c
       cmdstr = '/Xyps0 {dup '//
     1        '/Xpres exch cos Strlen mul Xpos0 add def'
       call filler
       cmdstr = '               '//
     1        '/Ypres exch sin Strlen mul Ypos0 add def} def'
       call filler
c
       cmdstr = '/Ypd {/Ypos exch def} def'
       call filler
c
       cmdstr = '/Ypjd '//
     1        ' {/Ypos exch Ypos exch Strlen mul sub def} def'
       call filler
c
       cmdstr = '/space ( ) def'
       call filler
c
       cmdstr = '%%EndProlog'
       call filler
c
       cmdstr = '%%Page: 1 1'
       call filler
c
c      "szrat" is the ratio of width to height of characters, determined
c  empirically to be: szrat = 0.6.  Set initial font to helvetica, 12 
c  point.
c
       szrat  = 0.6
       ifntsz = 12
       call setfnt ( 20 )
c
c      Set factor to 1 for initialization, reset later if CHOPIT called
c
       fac = 1.0
       call factor ( fac )
c
c      Set initial lineweight to 0.
c
       call setlw ( 0.0 )
c
c      Set initial grayscale to 0.
c
       call setgry ( 0.0 )
c
c      Set initial rgb colors to black(0).
c
       call setcolr ( 0.0, 0.0, 0.0 )
c
       if (.not. portrait) then
         cmdstr = '90 rotate 0 -8.5 inch translate '
         call filler
       endif
c
       call plot ( 18.0 / conver, 18.0 / conver, -3 )
c
       if ( portrait ) then
         xsh = 18.0 / conver
         ysh = 0.0
       else
         xsh = 0.0
         ysh = 18.0 / conver
       endif
       call plot ( xsh, ysh, -3 )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine setcolr ( red, green, blue )
c----------------------------------------------------------------------- 
c  Written by KEK: this routines sets the current colour
c
c  red, green, blue  rgb saturation ratios between 0 and 1
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          red     , green   , blue    , cred    , cgreen  
     1             , cblue   , cgry    , r       , g       , b
c
       common      / plt1     / cmdstr
       common      / colrcom  / cred    , cgreen  , cblue   , cgry
c
       external      filler
c----------------------------------------------------------------------- 
       r = amin1 ( 1.0, amax1 ( 0.0, red   ) )
       g = amin1 ( 1.0, amax1 ( 0.0, green ) )
       b = amin1 ( 1.0, amax1 ( 0.0, blue  ) )
c
       cmdstr = ' '
       write ( cmdstr, '(3F7.3,'' Sc'')' ) r, g, b
       call filler
       cred   = r
       cgreen = g
       cblue  = b
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine setfnt ( numfnt )
c----------------------------------------------------------------------- 
c  Written by KEK: This routines changes the typeface of the current 
c  font.
c
c  Modified by DAC (8/08): fonts described and rationalised
c
c  numfnt         Font            numfnt         Font
c     1    AvantGarde-Book           2    AvantGarde-BookOblique
c     3    AvantGarde-Demi           4    AvantGarde-DemiOblique
c     5    Bookman-Demi              6    Bookman-DemiItalic
c     7    Bookman-Light             8    Bookman-LightItalic
c     9    Courier-Bold             10    Courier-BoldOblique
c    11    Courier-Oblique          12    Courier
c    13    Helvetica-Bold           14    Helvetica-BoldOblique
c    15    Helvetica-Narrow-Bold    16    Helvetica-Narrow-BoldOblique
c    17    Helvetica-Narrow-Oblique 18    Helvetica-Narrow
c    19    Helvetica-Oblique        20    Helvetica
c    21    NewCenturySchlbk-Bold    22    NewCenturySchlbk-BoldItalic
c    23    NewCenturySchlbk-Italic  24    NewCenturySchlbk-Roman
c    25    Palatino-Bold            26    Palatino-BoldItalic
c    27    Palatino-Italic          28    Palatino-Roman
c    29    Symbol                   30    Times-Bold
c    31    Times-BoldItalic         32    Times-Italic
c    33    Times-Roman              34    ZapfChancery-MediumItalic
c    35    ZapfDingbats
c
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr  , scrc    , curfnt
       integer       numfnt  , ifntsz  , nfont
c
       character*40  fntnam  (  35)
c
       common      / fntcom   / curfnt  , ifntsz  , nfont
       common      / plt1     / cmdstr
c
       data          fntnam
     1             / 'AvantGarde-Book'
     2             , 'AvantGarde-BookOblique'
     3             , 'AvantGarde-Demi'
     4             , 'AvantGarde-DemiOblique'
     5             , 'Bookman-Demi'
     6             , 'Bookman-DemiItalic'
     7             , 'Bookman-Light'
     8             , 'Bookman-LightItalic'
     9             , 'Courier-Bold'
     1             , 'Courier-BoldOblique'
     2             , 'Courier-Oblique'
     3             , 'Courier'
     4             , 'Helvetica-Bold'
     5             , 'Helvetica-BoldOblique'
     6             , 'Helvetica-Narrow-Bold'
     7             , 'Helvetica-Narrow-BoldOblique'
     8             , 'Helvetica-Narrow-Oblique'
     9             , 'Helvetica-Narrow'
     1             , 'Helvetica-Oblique'
     2             , 'Helvetica'
     3             , 'NewCenturySchlbk-Bold'
     4             , 'NewCenturySchlbk-BoldItalic'
     5             , 'NewCenturySchlbk-Italic'
     6             , 'NewCenturySchlbk-Roman'
     7             , 'Palatino-Bold'
     8             , 'Palatino-BoldItalic'
     9             , 'Palatino-Italic'
     1             , 'Palatino-Roman'
     2             , 'Symbol'
     3             , 'Times-Bold'
     4             , 'Times-BoldItalic'
     5             , 'Times-Italic'
     6             , 'Times-Roman'
     7             , 'ZapfChancery-MediumItalic'
     8             , 'ZapfDingbats' /
c
       integer        lenstr
       external       filler  , lenstr
c----------------------------------------------------------------------- 
       nfont = numfnt
       if ( (numfnt .lt. 1) .or. (numfnt .gt. 35) ) then
         write (*,'(a)') 'Invalid font number encountered in **setfnt**'
         write (*,'(a)') 'Using Times-Roman default'
         nfont = 30
       endif
       scrc   = fntnam ( nfont )
       cmdstr = '/Curfnt /'//scrc(1:lenstr(scrc,132))//' findfont def'
       call filler
       write ( cmdstr, '(i3,'' Setf'')' ) ifntsz
       call filler
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine setgry ( gry )
c----------------------------------------------------------------------- 
c  Written by KEK: This routines sets the current gray level
c
c  gry         real between 0.0 and 1.0
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          gry     , cred    , cgreen  , cblue   , cgry
     1             , g
c
       common      / plt1     / cmdstr
       common      / colrcom  / cred    , cgreen  , cblue   , cgry
c
       external      filler
c----------------------------------------------------------------------- 
       g      = amax1 ( 0.0, amin1 ( 1.0, gry) )
       cmdstr = ' '
       write ( cmdstr, '(F7.3,'' Sg'')' ) g
       call filler
       cgry   = g
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine setlw ( rlwi )
c----------------------------------------------------------------------- 
c  Written by KEK: this routines sets the current linewidth.
c
c  rlwi        desired linewidth in inches
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
       real          rlwi    , curwid
c
       common      / plt1     / cmdstr
       common      / lcom     / curwid
c
       external      filler
c----------------------------------------------------------------------- 
       if ( abs(rlwi) .lt. 1.0e-5 ) then
         cmdstr = 'Slw0'
       else
         cmdstr = ' '
         write ( cmdstr, '(F7.3,'' Slw'')' ) rlwi
       endif
       call filler
       curwid = rlwi
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine sldlin ( x1, y1, x2, y2, thk )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine draws a solid line between specified 
c  points.
c
c  x1, y1      beginning coordinate of line
c  x2, y2      end coordinate of line
c  thk         thickness of the line (inches).  0.0 => line is drawn
c              with current line weight setting.  The width of the line 
c              is centred on the line joining the two points.
c----------------------------------------------------------------------- 
       implicit      none
c
       integer       iset
       real          x1      , y1      , x2      , y2      , thk
     1             , curwid  , curwids
c
       common      / lcom     / curwid
c
       external      setlw   , plot
c----------------------------------------------------------------------- 
       iset = 0
       if ( (thk .gt. 0.0) .and. (thk .ne. curwid) ) then
         iset    = 1
         curwids = curwid
         call setlw ( thk )
       endif
       call plot ( x1, y1, 3 )
       call plot ( x2, y2, 2 )
c
c      Reset linewidth to original value.
c
       if (iset .eq. 1) call setlw ( curwids )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine square ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by KEK: writes a square as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of square
c  radius      radius of square (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius  , rad
c
       real          xa      (   4), ya      (   4)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       rad   = 0.85 * radius
       xa(1) = xc - rad
       ya(1) = yc - rad
       xa(2) = xc + rad
       ya(2) = ya(1)
       xa(3) = xa(2)
       ya(3) = yc + rad
       xa(4) = xa(1)
       ya(4) = ya(3)
       call drwcrv ( xa, ya, 4, 0.0, .true. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine star ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by DAC, 6/10 using dk square as a template: writes a star 
c  as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of star
c  radius      radius of star (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius
c
       real          xa      (  10), ya      (  10)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       xa( 1) = xc
       ya( 1) = yc + radius
       xa( 2) = xc - 0.225 * radius
       ya( 2) = yc + 0.309 * radius
       xa( 3) = xc - 0.951 * radius
       ya( 3) = ya(2)
       xa( 4) = xc - 0.363 * radius
       ya( 4) = yc - 0.118 * radius
       xa( 5) = xc - 0.588 * radius
       ya( 5) = yc - 0.809 * radius
       xa( 6) = xc
       ya( 6) = yc - 0.382 * radius
       xa( 7) = xc + 0.588 * radius
       ya( 7) = ya(5)
       xa( 8) = xc + 0.363 * radius
       ya( 8) = ya(4)
       xa( 9) = xc + 0.951 * radius
       ya( 9) = ya(3)
       xa(10) = xc + 0.225 * radius
       ya(10) = ya(2)
       call drwcrv ( xa, ya, 10, 0.0, .true. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine stroke
c----------------------------------------------------------------------- 
c  Written by KEK: This routine ... 
c----------------------------------------------------------------------- 
       implicit      none
c
       character*132 cmdstr
c
       common      / plt1     / cmdstr
c
       external      filler
c----------------------------------------------------------------------- 
       cmdstr = 'stroke'
       call filler
c
       return
       end
c======================================================================= 
c=======================================================================
       subroutine subber ( titlein, nchar1, size, ang, fact )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine produces subscripted text.
c
c  titlein     character(s) to be subscripted
c  nchar1      number of characters to be subscripted
c  size        height of subscripted variable
c  ang         angle of text
c  fact        parameter for second level of subscripting
c----------------------------------------------------------------------- 
c  Modified by ABM, 10/2003: replaced Hollerith for character strings; 
c  generalized to accept special characters (of fonts 29 and 35).
c
c  Modified by DAC, 7/2010: introduced "fact" for second level of super- 
c  subscripting (fact = 1.0  = > no change from before)
c----------------------------------------------------------------------- 
       implicit      none
c
       character*(*) titlein
       character*132 cmdstr  , curfnt  , scrc
       character*80  titleb  , titlec
       character*1   bslash
       integer       nchar1  , ifntsz  , nfont   , nchar   , mchar   
     1             , numc    , m       , ireset  , iht     , ititle
       real          size    , ang     , fact    , conver  , pi      
     1             , angrad  , ss      , down    , xshift  , yshift  
c
       character*4   ctitle  (  20)
c
       common      / fntcom   / curfnt  , ifntsz  , nfont
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
c
       integer       lenstr
       external      filler  , blkstp  , lenstr  , setfnt
c----------------------------------------------------------------------- 
       bslash = char(92)
       pi     = acos (-1.0)
       angrad = ang * pi / 180.0
       ss     = 2.0 * size / 3.0
       down   = fact * ss / 2.0
       xshift = sin(angrad) * down
       yshift = cos(angrad) * down
c
       cmdstr = 'Xyp0d'
       call filler
c
       cmdstr = ' '
       write ( cmdstr, '(''/Xpos Xpres '',f8.2,'' add def '')' )
     1   xshift * conver
       call filler
       cmdstr = ' '
       write ( cmdstr, '(''/Ypos Ypres '',F8.2,'' sub def '')' )
     1   yshift * conver
       call filler
c
       if ( nchar1 .eq. -999 ) then
         ctitle(1) = titlein
         read ( ctitle(1), '(I4)' ) ititle
         nchar     = 1
       else
         nchar     = nchar1
         titlec    = titlein
         if ( iabs(nchar) .lt. 80 ) titlec(nchar+1:80) = ' '
         read ( titlec, '(20a4)' ) ctitle
       endif
c
c      Choose font proper height using currently set font
c
       mchar = iabs(nchar)
c
       if ( nchar1. eq. -999 ) then
         write ( titlec, '(A1,I10)' ) bslash, ititle
         call blkstp ( titlec, 80, titlec, mchar )
       else
         write ( titlec, '(20a4)' ) ctitle
c
c      Check if titlec contains (, ), or "backslash". These characters 
c  must be treated specially by preceding them with a "backslash". Do 
c  this to ( and ) even though they might be balanced, i.e. () within a 
c  string, which can be treated normally.
c
         titleb = titlec
         numc   = 0
         do 10 m=1,mchar
           if ( (titleb(m:m) .eq. '(') .or. (titleb(m:m) .eq. ')') .or.
     1          (titleb(m:m) .eq. bslash) ) then
             numc = numc + 1
             titlec(numc:numc) = bslash
           endif
           numc = numc + 1
           titlec(numc:numc) = titleb(m:m)
10       continue
         mchar = numc
       endif
c
       cmdstr = '('//titlec(1:mchar)//') Le'
c
c      Set character size
c
       iht = int ( ss * conver / 0.6 )     !0.6 factor is empirical
c
       ireset = 0
       if ( titlec(1:mchar) .ne. char(39) ) then
         if ( iht .ne. ifntsz ) then
           cmdstr = ' '
           write ( cmdstr, '(I3,'' Setf'')' ) iht
           call filler
           ifntsz = iht
         endif
       else        !Use prime, not apostrophe (do not change ifntsz)
         ireset = 1
         cmdstr = ' '
         write ( cmdstr
     1         ,'(''/Symbol findfont '',I3,'' scalefont setfont'')')iht
         call filler
       endif
c
       if ( titlec(1:mchar) .ne. char(39) ) then
         cmdstr = '('//titlec(1:mchar)//') Le'
       else
         cmdstr = '('//bslash//'242) Le'
       endif
c
c      Move pen to proper coordinates
c
       scrc   = 'xyd mv'
       cmdstr = cmdstr (1:lenstr(cmdstr,132))//' '
     1                 //scrc(1:lenstr(scrc,132))
c
c      Set angle
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(F7.1,'' rotate'')' ) ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                     //scrc(1:lenstr(scrc,132))
       endif
c
       if ( titlec(1:mchar) .ne. char(39) ) then
         scrc = '('//titlec(1:mchar)//') show'
       else
         scrc = '('//bslash//'242) show'
       endif
       cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                   //scrc(1:lenstr(scrc,132))
c
c      Reset angle.
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(F7.1,'' rotate'')') -ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                     //scrc(1:lenstr(scrc,132))
       endif
       call filler
c
c      Compute next character position.
c
       cmdstr = ' '
       write ( cmdstr, '(f6.1,'' Xyps0'')' ) ang
       call filler
c
c      Reset font if we drew a prime
c
       if ( ireset .eq. 1 ) call setfnt ( nfont )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine super ( titlein, nchar1, size, ang, fact )
c----------------------------------------------------------------------- 
c  Written by KEK: This routine produces superscripted text.
c
c  titlein     character(s) to be superscripted
c  nchar1      number of characters to be superscripted
c  size        height of superscripted variable
c  ang         angle of text
c  fact        parameter for second level of superscripting
c----------------------------------------------------------------------- 
c  Modified by ABM, 10/2003: replaced Hollerith for character strings; 
c  generalized to accept special characters (of fonts 29 and 35).
c
c  Modified by DAC, 7/2010: introduced "fact" for second level of super- 
c  subscripting (fact = 1.0  = > no change from before)
c----------------------------------------------------------------------- 
       implicit      none
c
       character*(*) titlein
       character*132 cmdstr  , curfnt  , scrc    
       character*80  titlec
       character*1   bslash
       integer       nchar1  , ifntsz  , nfont   , nchar   , mchar   
     1             , iht     , ireset  
       real          size    , ang     , fact    , conver  , pi      
     1             , angrad  , ss      , up      , xshift  , yshift 
c
       common      / fntcom   / curfnt  , ifntsz  , nfont
       common      / plt1     / cmdstr
       common      / cnvcom   / conver
c
       integer       lenstr
       external      filler  , lenstr  , setfnt
c----------------------------------------------------------------------- 
       bslash = char(92)
       titlec = titlein
c
c      If nchar1 is lt 0, we're calling from oversup and we should
c  lower superscript slightly
c
       nchar  = abs(nchar1)
       pi     = acos(-1.0)
       angrad = ang * pi/180.
       ss     = 2.0 * size / 3.0
       if ( nchar1 .ge. 0 ) then
         up = fact * 0.8 * ss
       else
         up = fact * 0.5 * ss
       endif
       xshift = sin(angrad) * up
       yshift = cos(angrad) * up
       cmdstr = 'Xyp0d'
       call filler
c
       cmdstr = ' '
       write ( cmdstr, '(''/Xpos Xpres '',f8.2,'' sub def '')' )
     1   xshift * conver
       call filler
       cmdstr = ' '
       write ( cmdstr, '(''/Ypos Ypres '',F8.2,'' add def '')' )
     1   yshift * conver
       call filler
c
c      Choose proper height.
c
       mchar = iabs(nchar)
c
c      Set character size.
c
       iht = int ( ss * conver / 0.6 )     !0.6 factor is empirical
c
       ireset = 0
       if ( titlec(1:mchar) .ne. char(39) ) then
         if ( iht .ne. ifntsz ) then
           cmdstr = ' '
           write ( cmdstr, '(I3,'' Setf'')' ) iht
           call filler
           ifntsz = iht
         endif
       else        !Use prime, not apostrophe (do not change ifntsz)
         ireset = 1
         cmdstr = ' '
         write ( cmdstr
     1         ,'(''/Symbol findfont '',I3,'' scalefont setfont'')') iht
         call filler
       endif
c
c      DRAW PRIME, NOT APOSTROPHE
c
       if ( titlec(1:mchar) .ne. char(39) ) then
         cmdstr = '('//titlec(1:mchar)//') Le'
       else
         cmdstr = '('//bslash//'242) Le'
       endif
c
c      Move pen to proper coordinates.
c
       scrc   = 'xyd mv'
       cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                   //scrc(1:lenstr(scrc,132))
c
c      Set angle.
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(F7.1,'' rotate'')' ) ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                     //scrc(1:lenstr(scrc,132))
       endif
c
       if ( titlec(1:mchar) .ne. char(39) ) then
         scrc = '('//titlec(1:mchar)//') show'
       else
         scrc = '('//bslash//'242) show'
       endif
       cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                   //scrc(1:lenstr(scrc,132))
c
c      Set angle.
c
       if ( ang .ne. 0.0 ) then
         scrc = ' '
         write ( scrc, '(F7.1,'' rotate'')' ) -ang
         cmdstr = cmdstr(1:lenstr(cmdstr,132))//' '
     1                     //scrc(1:lenstr(scrc,132))
       endif
       call filler
c
c      Compute next character position.
c
       cmdstr = ' '
       write ( cmdstr, '(f6.1,'' Xyps0'')' ) ang
       call filler
c
c      Reset font if we drew a prime.
c
       if ( ireset .eq. 1 ) call setfnt ( nfont )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine times ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by DAC, 8/08 using dk square as a template: writes a 
c  multiplication sign as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of multiplication sign
c  radius      radius of multiplication sign (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius  , rad
c
       real          xa      (2), ya      (2)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       rad   = 0.9 * radius
       xa(1) = xc - rad
       ya(1) = yc - rad
       xa(2) = xc + rad
       ya(2) = yc + rad
       call drwcrv ( xa, ya, 2, 0.0, .false. )
       ya(1) = yc + rad
       ya(2) = yc - rad
       call drwcrv ( xa, ya, 2, 0.0, .false. )
c
       return
       end
c======================================================================= 
c======================================================================= 
       subroutine triangle ( xc, yc, radius )
c----------------------------------------------------------------------- 
c  written by DAC, 8/08 using dk square as a template: writes a triangle
c  as a symbol on a 1-D plot
c
c  xc, yc      coordinates of centre of triangle
c  radius      radius of triangle (inches)
c----------------------------------------------------------------------- 
       implicit      none
c
       real          xc      , yc      , radius  
c
       real          xa      (3), ya      (3)
c
       external      drwcrv
c----------------------------------------------------------------------- 
       xa(1) = xc - radius
       ya(1) = yc - radius
       xa(2) = xc + radius
       ya(2) = ya(1)
       xa(3) = xc
       ya(3) = yc + radius
       call drwcrv ( xa, ya, 3, 0.0, .true. )
c
       return
       end
c======================================================================= 
